#include "engine.h"
#include "maps\hunter_map_overlay.h"

#define CACHE_ENABLE    asm("mov 2,r1 \n	ldsr r1,sr24": /* No Output */: /* No Input */: "r1" /* Reg r1 Used */)

#define CACHE_DISABLE    asm("ldsr r0,sr24")

/******************************************************************************/

void matrix_rotate_world(matrix_t m, int alp, int bet, int gam)
{
  int cosalp = fixcos(alp);
  int sinalp = fixsin(alp);
  int cosbet = fixcos(bet);
  int sinbet = fixsin(bet);
  int cosgam = fixcos(gam);
  int singam = fixsin(gam);

  m[0][0] =  fixmul(singam, fixmul(sinbet, sinalp)) + fixmul(cosgam, cosalp);
  m[0][1] =  fixmul(cosbet, sinalp);
  m[0][2] =  fixmul(singam, cosalp) - fixmul(cosgam, fixmul(sinbet, sinalp));

  m[1][0] =  fixmul(singam, fixmul(sinbet, cosalp)) - fixmul(cosgam, sinalp);
  m[1][1] =  fixmul(cosbet, cosalp);
  m[1][2] = -fixmul(cosgam, fixmul(sinbet, cosalp)) - fixmul(singam, sinalp);

  m[2][0] = -fixmul(singam, cosbet);
  m[2][1] =  sinbet;
  m[2][2] =  fixmul(cosgam, cosbet);
}

void matrix_rotate_object(matrix_t m, int alp, int bet, int gam)
{
  int cosalp = fixcos(alp);
  int sinalp = fixsin(alp);
  int cosbet = fixcos(bet);
  int sinbet = fixsin(bet);
  int cosgam = fixcos(gam);
  int singam = fixsin(gam);

  m[0][0] =  fixmul(cosalp, cosgam) - fixmul(sinalp, fixmul(sinbet, singam));
  m[0][1] =  fixmul(sinalp, cosgam) + fixmul(cosalp, fixmul(sinbet, singam));
  m[0][2] =  fixmul(cosbet, singam);

  m[1][0] = -fixmul(sinalp, cosbet);
  m[1][1] =  fixmul(cosalp, cosbet);
  m[1][2] = -sinbet;

  m[2][0] = -fixmul(cosalp, singam) - fixmul(sinalp, fixmul(sinbet, cosgam));
  m[2][1] =  fixmul(cosalp, fixmul(sinbet, cosgam)) - fixmul(sinalp, singam);
  m[2][2] =  fixmul(cosbet, cosgam);
}

/******************************************************************************/

/*
** Name: vertex_project
** Desc: Projects a vertex into screen space. (ie: 3D to 2D viewport space)
*/
void vertex_project(vertex_t* v)
{
  if (v->z > ENGINE_RECIPROCAL_MIN && v->z < ENGINE_RECIPROCAL_MAX)
  {
    int z = reciprocal[v->z];

    v->sx =  (((ENGINE_VIEWPORT_X2-ENGINE_VIEWPORT_X1)*v->x*z)>>9)+i2fdiv2(ENGINE_VIEWPORT_X2+ENGINE_VIEWPORT_X1);
    v->sy = -(((ENGINE_VIEWPORT_Y2-ENGINE_VIEWPORT_Y1)*v->y*z)>>9)+i2fdiv2(ENGINE_VIEWPORT_Y2+ENGINE_VIEWPORT_Y1);
  }
  else
  {
    v->sx = v->sy = 0;
  }
}

/******************************************************************************/

/*
** Name: draw_poly
** Desc: Renders a lit, texture mapped polygon to a viewport. No clipping is applied. 
*/
void draw_poly(vertex_t* p, int n, int texture, u32* const dst)
{
  int lt_i, lt_y, lt_yy;
  int rt_i, rt_y, rt_yy;
  int y1, y2, i, offset, cofs;
  u32 ofsetchunk;
  int lt_length = 0, rt_length = 0;
  int x1 = 999999, x2 = -999999;

  CACHE_ENABLE;

  for (i = n; i--;)
  {
    p[i].d = fixup(p[i].x);
    if (p[i].x < x1) {x1 = p[i].x; lt_i = rt_i = i;}
    if (p[i].x > x2) {x2 = p[i].x;}
  }
  CACHE_DISABLE;

  if ((x1 = fixup(x1)) < ENGINE_VIEWPORT_X1) x1 = ENGINE_VIEWPORT_X1;
  if ((x2 = fixup(x2)) > ENGINE_VIEWPORT_X2) x2 = ENGINE_VIEWPORT_X2;

  CACHE_ENABLE;
  for (; x1 < x2; x1++)
  {
    if (--lt_length <= 0)
    {
      do {
        i = lt_i++; if (lt_i >= n) lt_i = 0;
      } while ((lt_length = p[lt_i].d - p[i].d) <= 0);

      lt_yy = r2i((p[lt_i].y - (lt_y = p[i].y)) * reciprocal[lt_length]);
    }
    if (--rt_length <= 0)
    {
      do {
        i = rt_i--; if (rt_i < 0) rt_i = n - 1;
      } while ((rt_length = p[rt_i].d - p[i].d) <= 0);

      rt_yy = r2i((p[rt_i].y - (rt_y = p[i].y)) * reciprocal[rt_length]);
    }
    if ((y1 = fixup(lt_y)) < ENGINE_VIEWPORT_Y1) y1 = ENGINE_VIEWPORT_Y1;
    if ((y2 = fixup(rt_y)) > ENGINE_VIEWPORT_Y2) y2 = ENGINE_VIEWPORT_Y2;

    if ((i = y2 - y1) > 0)
    {
//-----------------------------------------------------------------
//bitstring renderer

	offset = (x1 << 4) + (y1 >> 4);
	ofsetchunk = (y1 & 0x0000000f) << 1;
	cofs = ((x1 ^ y1) & 1) ? 2 : 0;

	asm("
		st.w	r29,-4[r3]
		mov	%0,r26
		mov	%1,r27
		mov	%2,r28
		mov	%3,r29
		mov	%4,r30
		.hword 0x7C0B
		ld.w	-4[r3],r29
	"
	:  /* Output */
	: "r" (ofsetchunk), "r" (cofs), "r" (i << 1), "r" (&dst[offset]), "r" (&britered[texture][0]) /* Input */
	: "r26", "r27", "r28", "r29", "r30" /* Clobber */
	);
//-------------------------------------------------------------------------
    }
    lt_y += lt_yy;
    rt_y += rt_yy;
  }

  CACHE_DISABLE;

}

void set_pixel(int x, int y, int color, u32* const dst)
{
  int offset = (x << 4) + (y >> 4);
  int bitpos = (y & 0x0f) << 1;
  dst[offset] &= ~(0x03 << bitpos); //Clear pixel
  dst[offset] |= (color << bitpos); //Set new color
}

void draw_line(int x1, int y1, int x2, int y2, int color, u32* const dst)
{
  int dy, dx;
  int stepx, stepy;

  if ((x1 = fixup(x1)) < ENGINE_VIEWPORT_X1) x1 = ENGINE_VIEWPORT_X1;
  if (x1 >= ENGINE_VIEWPORT_X2) x1 = ENGINE_VIEWPORT_X2-1;
  if ((x2 = fixup(x2)) < ENGINE_VIEWPORT_X1) x2 = ENGINE_VIEWPORT_X1;
  if (x2 >= ENGINE_VIEWPORT_X2) x2 = ENGINE_VIEWPORT_X2-1;
  if ((y1 = fixup(y1)) < ENGINE_VIEWPORT_Y1) y1 = ENGINE_VIEWPORT_Y1;
  if (y1 >= ENGINE_VIEWPORT_Y2) y1 = ENGINE_VIEWPORT_Y2-1;
  if ((y2 = fixup(y2)) < ENGINE_VIEWPORT_Y1) y2 = ENGINE_VIEWPORT_Y1;
  if (y2 >= ENGINE_VIEWPORT_Y2) y2 = ENGINE_VIEWPORT_Y2-1;

  dy = y2 - y1;
  dx = x2 - x1;

  if (dy < 0) { dy = -dy;  stepy = -1; } else { stepy = 1; }
  if (dx < 0) { dx = -dx;  stepx = -1; } else { stepx = 1; }
  dy <<= 1;
  dx <<= 1;

  set_pixel(x1, y1, color, dst);
  if (dx > dy) {
    int fraction = dy - (dx >> 1);
    while (x1 != x2) {
      if (fraction >= 0) {
        y1 += stepy;
        fraction -= dx;
      }
      x1 += stepx;
      fraction += dy;
      set_pixel(x1, y1, color, dst);
    }
  } else {
    int fraction = dx - (dy >> 1);
    while (y1 != y2) {
      if (fraction >= 0) {
        x1 += stepx;
        fraction -= dy;
      }
      y1 += stepy;
      fraction += dx;
      set_pixel(x1, y1, color, dst);
    }
  }
}

void draw_map_overlay(engine_t* engine)
{
  int x;

  CACHE_ENABLE;
  for (x = 0; x < 128; x++)
  {
//-----------------------------------------------------------------
//Map overlay bitstring renderer

	asm("
		st.w	r29,-4[r3]
		mov	0,r26
		mov	0,r27
		mov	%0,r28
		mov	%1,r29
		mov	%2,r30
		.hword 0x7C09
		mov	%0,r28
		mov	%1,r29
		mov	%3,r30
		.hword 0x7C08
		ld.w	-4[r3],r29
	"
	:  /* Output */
	: "r" (256), "r" (&engine->baffer[x << 4]), "r" (&hunter_map_overlay_mask[x][0]), "r" (&hunter_map_overlay[x][0]) /* Input */
	: "r26", "r27", "r28", "r29", "r30" /* Clobber */
	);
//-------------------------------------------------------------------------
  }
  CACHE_DISABLE;

  set_pixel(f2i(engine->main_entity->x), 127 - f2i(engine->main_entity->y), 3, engine->baffer);
}

/******************************************************************************/

/*
** Name: polygon_clip
** Desc: clip a polygon to pre-calculated plane distances.
*/
#ifndef __PATCH_POLYGON_CLIP__
int polygon_clip(engine_t* engine, vertex_t** dst, vertex_t** src, int n)
{
  vertex_t** tmp = dst;
  int isLine = (n == 2);

  if (n > 1)
  {
    vertex_t* pre;

    CACHE_ENABLE;
    for (pre = src[n - 1]; n--; pre = *src++)
    {
      if (pre->d >= 0) *dst++ = pre;

      if ((pre->d ^ src[0]->d) < 0 && (isLine ? pre->d < 0 : 1))
      {
        vertex_t* dv = *dst++ = ENGINE_VERTEX(engine);
        vertex_t* sv = src[0];
        int d = pre->d - sv->d;
        int r = f2i(d < 0 ? reciprocal[-d] * -pre->d : reciprocal[d] * pre->d);

        dv->x = fixlerp(pre->x, sv->x, r);
        dv->y = fixlerp(pre->y, sv->y, r);
        dv->z = fixlerp(pre->z, sv->z, r);

        vertex_project(dv);
      }
    }
    CACHE_DISABLE;
  }
  return dst - tmp;
}
#endif

/******************************************************************************/

/*
** Name: draw_clipped_poly
** Desc: Renders a clipped poly to a screen. The texture is DDA mapped to the given polygon.
*/
#ifndef __PATCH_DRAW_CLIPPED_POLY__
void draw_clipped_poly(engine_t* engine, polyclip_t src, int n, int tid)
{
  int i;
  polygon_t p;

  CACHE_ENABLE;
  for (i = n; i--;)
  {
    p[i].x = src[i]->sx;
    p[i].y = src[i]->sy;
  }
  CACHE_DISABLE;

  draw_poly(p, n, tid, engine->baffer);
}
#endif

/******************************************************************************/

/*
** Name: draw_texture
** Desc: Renders a texture to a screen. The texture is DDA mapped to the given polygon.
*/
#ifndef __PATCH_DRAW_TEXTURE__
void draw_texture(engine_t* engine, polyclip_t p, int n, int tid)
{
  int i;
  int norg = n;

  for (i = n; i--;)
  {
    if (
       (p[i]->x + p[i]->z) < 0 ||
       (p[i]->z - p[i]->x) < 0 ||
       (p[i]->y + p[i]->z) < 0 ||
       (p[i]->z - p[i]->y) < 0 )
    {
      polyclip_t a, b;

      for (i = n; i--; p[i]->d = p[i]->x + p[i]->z); n = polygon_clip(engine, a, p, n); // Left
      for (i = n; i--; a[i]->d = a[i]->z - a[i]->x); n = polygon_clip(engine, b, a, n); // Right
      for (i = n; i--; b[i]->d = b[i]->y + b[i]->z); n = polygon_clip(engine, a, b, n); // Top
      for (i = n; i--; a[i]->d = a[i]->z - a[i]->y); n = polygon_clip(engine, b, a, n); // Bottom

      p = b;
      
      break;
    }
  }
  if (n > 2) draw_clipped_poly(engine, p, n, tid);
  
  else if (n == 2 && norg == 2)
    draw_line(p[0]->sx, p[0]->sy, p[1]->sx, p[1]->sy, tid, engine->baffer);
}
#endif
/******************************************************************************/

/*
** Name: rotate_vector
** Desc: Rotates a cell vector and apply world lighting. Rotated vectors are
**       cached in map cells so they can be reused. The rotated vertex is
**       indexed by its un-rotated y position.
*/
vertex_t* rotate_vector(engine_t* engine, vec3_t* v)
{
  int x, y, z;
  vertex_t* rv;
  
  x = v->x - engine->camera->x;
  y = v->y - engine->camera->z;
  z = v->z - engine->camera->y;

  rv    = ENGINE_VERTEX(engine);
  rv->x = f2i(engine->m[0][0] * x + engine->m[0][1] * y + engine->m[0][2] * z);
  rv->y = engine->is2d ? y : f2i(engine->m[1][0] * x + engine->m[1][1] * y + engine->m[1][2] * z);
  rv->z = f2i(engine->m[2][0] * x + engine->m[2][1] * y + engine->m[2][2] * z);

  vertex_project(rv);

  return rv;
}

/*
** Name: draw_quad
** Desc: Renders a single quad/rectangle of a cell using the given texture
**       mapping mode and texture id. The mapping mode changes depending
**       of if this is a north-south wall, east-west wall or ceiling/floor.
*/
void draw_quad(engine_t* engine, int tid, quad_t quad)
{
  polyclip_t p;

  p[0] = rotate_vector(engine, &quad[0]);
  p[1] = rotate_vector(engine, &quad[1]);
  p[2] = rotate_vector(engine, &quad[2]);
  p[3] = rotate_vector(engine, &quad[3]);

  draw_texture(engine, p, 4, tid);
}

/******************************************************************************/

/*
** Name: draw_entity_as_model
** Desc: Draws a entity 3D model at the given location.
*/
void draw_entity_as_model(entity_t* entity)
{
  s8* model = entity->visual.data;
  engine_t* engine = entity->engine;
  matrix_t m;
  polyclip_t p;
  vec3_t verts[100];
  int i, j, nvertices;

  model++;

  matrix_rotate_object(m, f2i(entity->r), f2i(entity->p), f2i(entity->t));
  
  for (nvertices = *model++, i = 0; i < nvertices; i++)
  {
    int u = *model++;
    int v = *model++;
    int w = *model++;
    int x = f2i(m[0][0] * u + m[0][1] * v + m[0][2] * w) + entity->x - engine->camera->x;
    int y = f2i(m[1][0] * u + m[1][1] * v + m[1][2] * w) + entity->z - engine->camera->z;
    int z = f2i(m[2][0] * u + m[2][1] * v + m[2][2] * w) + entity->y - engine->camera->y;
    
    verts[i].x = f2i(engine->m[0][0] * x + engine->m[0][1] * y + engine->m[0][2] * z);
    verts[i].y = engine->is2d ? y : f2i(engine->m[1][0] * x + engine->m[1][1] * y + engine->m[1][2] * z);
    verts[i].z = f2i(engine->m[2][0] * x + engine->m[2][1] * y + engine->m[2][2] * z);
  }
  for (i = *model++; i--;)
  {
    int npts = *model++;
    for (j = 0; j < npts; j++)
    {
      p[j] = ENGINE_VERTEX(engine);
      *((vec3_t*)&p[j][0]) = verts[*model++];
      p[j]->u = i2f(*model++);
      p[j]->v = i2f(*model++);
      p[j]->l = i2f(48);
      vertex_project(p[j]);
    }
    draw_texture(engine, p, npts, *model++);
  }
}

/******************************************************************************/

void engine_draw(engine_t* engine)
{
  cell_t* cell;
  rom_cell_t* cello;
  quad_t q, q1, q2;
  vec3_t *p, *p1, *p2;
  entity_t* e1;
  entity_t* e2;
  int i, j, x, y;
  h_cell_t* vcells[6][11];
  h_cell_t** hc;
  h_cell_t** lhc;
  int onrow[6] = {0,4,6,8,10,10};
  entity_t rent;
  int main_entity_drawn = 0;
  s8 incdec = engine->main_entity->x & 0x80?-1:1;

  CLEARMEM(&rent);
  rent.engine = engine;

  p = (vec3_t*)&q;
  p2 = 0;

  engine->is2d = !engine->camera->p && !engine->camera->r;
  
  matrix_rotate_world(engine->m,
    -f2i(engine->camera->r),
    -f2i(engine->camera->p),
    -f2i(engine->camera->t));  

  // Top left xy
  x = (engine->main_entity->x & 0xFFFFFF00) - 1024/*i2f(4)*/;
  y = (engine->main_entity->y & 0xFFFFFF00) + 768/*i2f(3)*/;

  if (!(engine->main_entity->x & 0x80))
    x -= 256;

  for (j = 5; j >= 0; j--)
    for (i = 0; i < 11; i++)
      vcells[j][i] = (h_cell_t*)&huntercells[engine->grid->cells[f2i(y)-5+j][f2i(x)+i]];

  for (j = 5; j >= 0; j--)
  {
    int start = 10 - onrow[j] >> 1;
    lhc = hc;
    for (i = 0; i < onrow[j]; i++)
    {
      hc = &vcells[j][start+i];

      // Cell Bottom (Floor)
      if ((*hc)->color != 1)
      {
        p[0].x = p[3].x = x + ((start+i) << 8); p[1].x = p[2].x = p[0].x+256;
        p[2].z = p[3].z = y - ((5-j)     << 8); p[0].z = p[1].z = p[3].z+256;
        p[0].y = (*hc)->height; p[1].y = (*(hc+1))->height;
        p[2].y = (*(hc-10))->height; p[3].y = (*(hc-11))->height;

        if ((*hc)->style < 4)
        {
          p[(*hc)->style].x = p[0].x + 128;
          p[(*hc)->style].z = p[3].z + 128;
        }

        draw_quad(engine, (*hc)->color, p);

        if (hc == &vcells[1][4])
        {
          p1 = p;
          p = (vec3_t*)&q1;
        }
        else if (hc == &vcells[1][5])
        {
          p2 = p;
          p = (vec3_t*)&q2;
        }
      }
    }

    if (j < 5)
    {
      if (incdec > 0)
        lhc -= (onrow[j+1]-1);

      for (i = onrow[j+1]; i > 0; i--, lhc+=incdec)
      {
        u16 currgrid = ((f2i(y)-4+j) << 7) + f2i(x) + (10-onrow[j+1]>>1) + (incdec>0?onrow[j+1]-i:i-1);
        entity_t* pent = (entity_t*)search_binary_tree(engine->root, currgrid);

        if ((*lhc)->entityID >= 0)
        {
          int x1, x2;
          rom_entity_t* prent = (rom_entity_t*)&hunterentities[(*lhc)->entityID];

          do
          {
            if (engine->building_strength[prent - hunterentities] == 0)
              continue;

            rent.x = prent->x;
            rent.y = prent->y;

            x1 = (*(lhc-11))->height + (rent.y * ((*lhc)->height - (*(lhc-11))->height) >> 8);
            x2 = (*(lhc-10))->height + (rent.y * ((*(lhc+1))->height - (*(lhc-10))->height) >> 8);
            rent.z = x1 + (rent.x * (x2 - x1) >> 8) - ((s8*)prent->model)[3];
            rent.x += x + ((lhc - &vcells[j+1][0]) << 8);
            rent.y += y - ((4-j) << 8);

            rent.visual.data = prent->model;

            while (pent && pent->grid == currgrid && pent->y > rent.y) {
              draw_entity_as_model(pent);
              pent = pent->right;
            }

            draw_entity_as_model(&rent);
          } while ((prent++)->more > 0);
        }
        while (pent && pent->grid == currgrid) {
          draw_entity_as_model(pent);
          pent = pent->right;
        }
      }
      if (j == 1)
      {
        h_cell_t** mainc = &vcells[2][4 + (engine->main_entity->x & 0x80?0:1)];
        
        if ((*(mainc-11))->height > (*mainc)->height && (*(mainc-11))->height > (*(mainc-22))->height)
        {
          draw_quad(engine, vcells[1][4]->color, p1);
          if (p2)
            draw_quad(engine, vcells[1][5]->color, p2);
        }
      }
    }
  }

}
