#ifndef __ENGINE_H__
#define __ENGINE_H__
#include "include\types.h"

/******************************************************************************/

#ifdef __cplusplus
extern "C"{
#endif

/******************************************************************************/

#ifndef TRUE
#define TRUE (1)
#endif

#ifndef FALSE
#define FALSE (0)
#endif

#define ABS(A) ((A)<0?-(A):(A))
#define MIN(A,B) ((A)<(B)?(A):(B))
#define MAX(A,B) ((A)>(B)?(A):(B))

#define CLEARMEM(A) engine_memset(A, 0, sizeof(*(A)))

/******************************************************************************/

#define ENGINE_ENTITY_MAX 80
#define ENGINE_POLYGON_MAX 16
#define ENGINE_VPOOL_MAX 512
#define ENGINE_BULLET_MAX 16
#define ENGINE_RAY_MAX 40
#define ENGINE_RAY_QUALITY 1
#define ENGINE_CELL_MAX 32
#define ENGINE_MAP_WIDTH 64
#define ENGINE_MAP_HEIGHT 64
#define ENGINE_LIGHT_MIN i2f(1)
#define ENGINE_LIGHT_MAX i2f(63)
#define ENGINE_RECIPROCAL_MIN 0
#define ENGINE_RECIPROCAL_MAX 16384
#define ENGINE_SINTABLE_MAX 2048
#define ENGINE_TRIANGLE_BUCKET_MAX 512

/******************************************************************************/

#define FIXONE  0x100
#define FIXHALF 0x80
#define FIXCEIL 0xFF

#define f2i(A) ((A)>>8)
#define i2f(A) ((A)<<8)
#define r2i(A) ((A)>>16)
#define i2fdiv2(A) ((A)<<7)
#define f2fl(A) ((A)/256.0)
#define fl2f(A) ((A)*256.0)
#define fixtrunc(A) i2f(f2i(A))
#define fixlerp(A,B,C) ((A)+fixmul((B)-(A),(C)))
#define fixup(A) f2i((A)+FIXCEIL)
#define fixinv(A) fixdiv(FIXUNIT,(A))

#define intdiv(A,B) ((A)/(B))
#define muldiv(A,B,C) intdiv((A)*(B),(C))
#define fixmul(A,B) f2i((A)*(B))
#define fixdiv(A,B) intdiv(i2f(A),(B))
#define fixsin(A) sintable[(A)&2047]
#define fixcos(A) fixsin((A)+512)

/******************************************************************************/

#define ENGINE_VIEWPORT_INTERVAL 14
#define ENGINE_VIEWPORT_INTERVAL_ERROR 7

#define ENGINE_VIEWPORT_X1 0
#define ENGINE_VIEWPORT_Y1 0
#define ENGINE_VIEWPORT_X2 384
#define ENGINE_VIEWPORT_Y2 189
#define ENGINE_VIEWPORT_WIDTH 384
#define ENGINE_VIEWPORT_HEIGHT 189

#define ENGINE_TEXGEN_WALL_LR  0
#define ENGINE_TEXGEN_WALL_FB  1
#define ENGINE_TEXGEN_FLAT     2
#define ENGINE_TEXGEN_BASIC    3

typedef unsigned char byte;
/******************************************************************************/

#define AFFINE_CASE(I) case (I): AFFINE(0); fb++;

#define AFFINE_LOOP  \
for (; i >= 16; i -= 16, fb += 16) { \
AFFINE( 0) AFFINE( 1) AFFINE( 2) AFFINE( 3) AFFINE( 4) AFFINE( 5) AFFINE( 6) AFFINE( 7) \
AFFINE( 8) AFFINE( 9) AFFINE(10) AFFINE(11) AFFINE(12) AFFINE(13) AFFINE(14) AFFINE(15) \
} switch (i) { \
AFFINE_CASE(16) AFFINE_CASE(15) AFFINE_CASE(14) AFFINE_CASE(13) \
AFFINE_CASE(12) AFFINE_CASE(11) AFFINE_CASE(10) AFFINE_CASE( 9) \
AFFINE_CASE( 8) AFFINE_CASE( 7) AFFINE_CASE( 6) AFFINE_CASE( 5) \
AFFINE_CASE( 4) AFFINE_CASE( 3) AFFINE_CASE( 2) AFFINE_CASE( 1) \
}

/******************************************************************************/

typedef struct
{
  int x, y;
} vec2_t;

typedef struct
{
  int x, y, z;
} vec3_t;

typedef int matrix_t[3][3];
typedef vec3_t quad_t[4];

/******************************************************************************/

typedef struct
{
  u16 pixels[ENGINE_VIEWPORT_HEIGHT][ENGINE_VIEWPORT_WIDTH];
} framebuffer_t;

typedef struct
{
  framebuffer_t* front;
  framebuffer_t* back;
} viewport_t;

/******************************************************************************/

// Type definition of a polygon vertex
typedef struct  
{ 
  s32 x;  // 24:8 Fixed Point X Position.
  s32 y;  // 24:8 Fixed Point Y Position.
  s32 z;  // 24:8 Fixed Point Z Position.
  s32 l;  // 24:8 Fixed point lighting. Ranges from 0 to 63.
  s32 u;  // 24:8 texture 'u' cooridinate.
  s32 v;  // 24:8 texture 'v' cooridinate.
  s32 d;  // Distance to plane value. Also used for vertex scanline storage.
  s32 sx; // X projected into screen space.
  s32 sy; // Y projected into screen space.
} vertex_t;

typedef vertex_t polygon_t[ENGINE_POLYGON_MAX];
typedef vertex_t* polyclip_t[ENGINE_POLYGON_MAX];

/******************************************************************************/

typedef struct
{
  void* data;
  u8 width;
  u8 height;
  u8 mode;
} entity_visual_t;

typedef struct entity_t
{
  struct entity_t* parent;
  struct entity_t* left;
  struct entity_t* right;
  
  void* engine;
  
  entity_visual_t visual;

  u8 swi;
  s8 life;
  u16 radius;
  u16 grid;
  s32 x, xx;  // X position and velocity.
  s32 y, yy;  // Y position and velocity.
  s32 z, zz;  // Z position and velocity.
  s32 p, pp;  // Pitch
  s32 r, rr;  // Roll
  s32 t, tt;  // Turn
  s32 v;
  s32 rx;
  s32 ry;
  s32 rz;
  void (*ontick)(struct entity_t* e);
  void (*onhit)(struct entity_t* e1, struct entity_t* e2);
  void (*onBGhit)(struct entity_t* e, u16 rom_entity_ID);
  void (*ondraw)(struct entity_t* e);
} entity_t;

typedef void (*entity_behaviour_t)(entity_t* e);

/******************************************************************************/

typedef struct
{
  u8 color;
  u16 height;
  u8 style;
  s16 entityID;
} h_cell_t;

typedef struct
{
  u16 cells[128][128];
} h_map_t;

typedef struct
{
  u8 x;
  u8 y;
  void* model;
  u8 more;
} rom_entity_t;

typedef struct
{
  u8 swi;   // Cell switchs (flags).
  u8 ent;   // Entity Type. Used for setting start locations for entities.  
  s16 top;  // 8:8 fixed point cell top value.
  s16 mid;  // 8:8 fixed point cell mid value. (Currently Unused/Reserved)
  s16 bot;  // 8:8 fixed point cell bot value.
  u16 lit;  // Cell light setting.
  u8 wtx;   // Cell wall texture id.
  u8 ttx;   // Cell top texture id.
  u8 mtx;   // Cell mid texture id. (Currently Unused/Reserved)
  u8 btx;   // Cell bot texture id.
  u8 res1;  // Reserved.
  u8 res2;  // Reserved.
} rom_cell_t;

typedef struct
{
  u8 name[32];
  u8 auth[32];
  u8 desc[64];
  rom_cell_t cells[ENGINE_MAP_HEIGHT][ENGINE_MAP_WIDTH];
} rom_map_t;

typedef struct 
{
  u32 mark;
  entity_t* entities;
} cell_t;

/******************************************************************************/

typedef struct
{
  u16 pad, _pad;
} keyboard_t;

/******************************************************************************/

typedef struct
{
  int gravity;    // Gravity force.
  int jumping;    // Jumping force.
  int crawling;   // Crawling height.
  int walking;    // Walking height.
  int ceiling;    // Crouch distance.
} game_t;

/******************************************************************************/

typedef struct
{
  game_t game;

  u32 tick;
  u32 mark;
  u32 fps;
  entity_t* camera;
  entity_t* main_entity;
  u8 currModel;
  u8 mapon;
  u16 camera_zoom;
  u16 camera_zoom_to;

  viewport_t viewport;
  keyboard_t keyboard;

  u32* scrin;
  u32* baffer;

  int is2d;
  matrix_t m;

  u8 building_strength[100];
  int nentities;
  entity_t entities[ENGINE_ENTITY_MAX];
  entity_t* root;

  int nvpool;
  vertex_t vpool[ENGINE_VPOOL_MAX];

  vec2_t ray_pos[ENGINE_RAY_MAX];
  vec2_t ray_vel[ENGINE_RAY_MAX];
  vec2_t vis[ENGINE_CELL_MAX];
  cell_t cells[ENGINE_MAP_HEIGHT][ENGINE_MAP_WIDTH];
  rom_map_t* curr_map;
  h_map_t* grid;

} engine_t;

#define ENGINE_VERTEX(ENGINE) (&(ENGINE)->vpool[(ENGINE)->nvpool++ & 511])
#define ENGINE_BULLET(ENGINE) ((ENGINE)->bullets[(ENGINE)->nbullets++ & 15])

/******************************************************************************/

extern void entity_motion(entity_t* e);

/******************************************************************************/

extern void engine_draw(engine_t* engine);
extern void draw_texture(engine_t* engine, polyclip_t src, int n, int tid);
extern void draw_poly(vertex_t* p, int n, const int texture, u32* const dst);
extern int polygon_clip(engine_t* engine, vertex_t** dst, vertex_t** src, int n);
extern void draw_clipped_poly(engine_t* engine, polyclip_t src, int n, int tid);

/******************************************************************************/

extern void engine_clear_entities(engine_t* engine);
extern void engine_load_map(engine_t* engine, rom_map_t* map, h_map_t* hsrc);
extern void engine_init_map(engine_t* engine);
extern void engine_init(engine_t* engine,
  framebuffer_t* front,
  framebuffer_t* back);
extern void engine_tick(engine_t* engine);
extern void engine_putc(int x, int c);
extern void engine_puts(int x, char* s);
extern entity_t* engine_entity(engine_t* engine, int x, int y, int z, entity_behaviour_t behaviour);

void matrix_rotate_world(matrix_t m, int alp, int bet, int gam);
void matrix_rotate_object(matrix_t m, int alp, int bet, int gam);
void vertex_project(vertex_t* v);

/******************************************************************************/

void* engine_memcpy(void* d, void* s, unsigned n);
void* engine_memset(void *s, int c, unsigned n);

/******************************************************************************/

extern const rom_map_t e1m1;
extern const h_map_t huntermap;
extern const h_cell_t huntercells[];
extern const rom_entity_t hunterentities[];

/******************************************************************************/

extern const u32 britered[10][16];
u8 lajt;
extern const int reciprocal[ENGINE_RECIPROCAL_MAX];
extern const int sintable[ENGINE_SINTABLE_MAX];

/******************************************************************************/

#ifdef __cplusplus
} // extern "C"
#endif

#endif


