#include "game.h"

/******************************************************************************/

const s8 cube[] =
{
  1,              // Number of vertex lists (Animation Frames)
  8,              // Number of verices per list.
   127,  -127,  127, // Vertex X, Y, Z. Range (-120..120)
   127,  -127, -127,
  -127,  -127, -127,
  -127,  -127,  127,
   127,   127,  127,
   127,   127, -127,
  -127,   127, -127,
  -127,   127,  127,
  6,              // Number of faces in the polygon mesh.
                  // Each face is (NPoints, VID, U, V) + texture id.
  4, 7, 0, 63, 4, 63, 63, 5, 63, 0, 6, 0, 0, 9,
  4, 6, 0, 63, 5, 63, 63, 1, 63, 0, 2, 0, 0, 2,
  4, 5, 0, 63, 4, 63, 63, 0, 63, 0, 1, 0, 0, 7,
  4, 7, 0, 63, 3, 63, 63, 0, 63, 0, 4, 0, 0, 2,
  4, 7, 0, 63, 6, 63, 63, 2, 63, 0, 3, 0, 0, 7,
  4, 3, 0, 63, 2, 63, 63, 1, 63, 0, 0, 0, 0, 9
};

const s8 car[] =
{
  1,              // Number of vertex lists (Animation Frames)
  42,              // Number of verices per list.
   18,  -41,   33, //  0 Right front shadow
   18,   -4,  -30, //  1 Right rear boot
   18,    0,   14, //  2 Right front window bottom
   18,   -4,   30, //  3 Right front hood
   18,  -27,   33, //  4 Right front bottom
   14,   19,  -20, //  5 Right rear roof
   14,   19,    7, //  6 Right front roof

  -18,  -27,  -33, //  7 Left rear bottom
  -18,   -4,  -30, //  8 Left rear boot
  -18,    0,   14, //  9 Left front window bottom
  -18,   -4,   30, //  10 Left front hood
  -18,  -27,   33, //  11 Left front bottom
  -14,   19,  -20, //  12 Left rear roof
  -14,   19,    7, //  13 Left front roof

   18,  -41, -33, //  14 Right rear shadow
  -18,  -41, -33, //  15 Left rear shadow
  -18,  -41,  33, //  16 Left front shadow
   18,  -27, -33, //  17 Right rear bottom

  18,  -27,  -30, //  18 Right rear wheel rear middle
  18,  -14,  -26, //  19 Right rear wheel rear top
  18,  -14,  -18, //  20 Right rear wheel front top
  18,  -27,  -14, //  21 Right rear wheel front middle
  18,  -41,  -18, //  22 Right rear wheel front bottom
  18,  -41,  -26, //  23 Right rear wheel rear bottom

  18,  -27,  14, //  24 Right front wheel rear middle
  18,  -14,  18, //  25 Right front wheel rear top
  18,  -14,  26, //  26 Right front wheel front top
  18,  -27,  30, //  27 Right front wheel front middle
  18,  -41,  26, //  28 Right front wheel front bottom
  18,  -41,  18, //  29 Right front wheel rear bottom

  -18,  -27,  -30, //  30 Left rear wheel rear middle
  -18,  -14,  -26, //  31 Left rear wheel rear top
  -18,  -14,  -18, //  32 Left rear wheel front top
  -18,  -27,  -14, //  33 Left rear wheel front middle
  -18,  -41,  -18, //  34 Left rear wheel front bottom
  -18,  -41,  -26, //  35 Left rear wheel rear bottom

  -18,  -27,  14, //  36 Left front wheel rear middle
  -18,  -14,  18, //  37 Left front wheel rear top
  -18,  -14,  26, //  38 Left front wheel front top
  -18,  -27,  30, //  39 Left front wheel front middle
  -18,  -41,  26, //  40 Left front wheel front bottom
  -18,  -41,  18, //  41 Left front wheel rear bottom

  19,              // Number of faces in the polygon mesh.
                  // Each face is (NPoints, VID, U, V) + texture id.
  4, 14, 0, 0, 15, 0, 0, 16, 0, 0, 0, 0, 0, 0, // Shadow
  4, 6, 0, 0, 13, 0, 0, 9, 0, 0, 2, 0, 0, 0, // Front window
  4, 2, 0, 0, 9, 0, 0, 10, 0, 0, 3, 0, 0, 9, // Hood
  4, 3, 0, 0, 10, 0, 0, 11, 0, 0, 4, 0, 0, 2, // Front
  4, 5, 0, 0, 12, 0, 0, 13, 0, 0, 6, 0, 0, 9, // Roof
  4, 1, 0, 0, 8, 0, 0, 12, 0, 0, 5, 0, 0, 0, // Rear window
  4, 17, 0, 0, 7, 0, 0, 8, 0, 0, 1, 0, 0, 2, // Rear
  5, 17, 0, 0, 1, 0, 0, 2, 0, 0, 3, 0, 0, 4, 0, 0, 7, // Right side
  4, 1, 0, 0, 5, 0, 0, 6, 0, 0, 2, 0, 0, 8, // Right window
  5, 7, 0, 0, 11, 0, 0, 10, 0, 0, 9, 0, 0, 8, 0, 0, 7, // Left side
  4, 8, 0, 0, 9, 0, 0, 13, 0, 0, 12, 0, 0, 8, // Left window
  4, 18, 0, 0, 19, 0, 0, 22, 0, 0, 23, 0, 0, 5, // Right rear wheel light part
  4, 19, 0, 0, 20, 0, 0, 21, 0, 0, 22, 0, 0, 2, // Right rear wheel dark part
  4, 24, 0, 0, 25, 0, 0, 26, 0, 0, 29, 0, 0, 2, // Right front wheel dark part
  4, 26, 0, 0, 27, 0, 0, 28, 0, 0, 29, 0, 0, 5, // Right front wheel light part
  4, 30, 0, 0, 35, 0, 0, 34, 0, 0, 31, 0, 0, 5, // Left rear wheel light part
  4, 31, 0, 0, 34, 0, 0, 33, 0, 0, 32, 0, 0, 2, // Left rear wheel dark part
  4, 36, 0, 0, 41, 0, 0, 38, 0, 0, 37, 0, 0, 5, // Left front wheel light part
  4, 38, 0, 0, 41, 0, 0, 40, 0, 0, 39, 0, 0, 2 // Left front wheel dark part
};

const s8 jeep[] =
{
  1,              // Number of vertex lists (Animation Frames)
  60,              // Number of verices per list.
   27,  -67,   39, //  0 Right front shadow
   27,   -9,  -39, //  1 Right rear boot
   27,   -5,   12, //  2 Right front window bottom
   27,   -9,   39, //  3 Right front hood
   27,  -47,   39, //  4 Right front bottom
   23,   30,  -36, //  5 Right rear roof
   23,   27,    7, //  6 Right front roof

  -27,  -47,  -39, //  7 Left rear bottom
  -27,   -9,  -39, //  8 Left rear boot
  -27,   -5,   12, //  9 Left front window bottom
  -27,   -9,   39, //  10 Left front hood
  -27,  -47,   39, //  11 Left front bottom
  -23,   30,  -36, //  12 Left rear roof
  -23,   27,    7, //  13 Left front roof

   27, -67, -39, //  14 Right rear shadow
  -27, -67, -39, //  15 Left rear shadow
  -27, -67,  39, //  16 Left front shadow
   27, -47, -39, //  17 Right rear bottom

  27,  -47,  -34, //  18 Right rear wheel rear middle
  27,  -32,  -31, //  19 Right rear wheel rear top
  27,  -27,  -24, //  20 Right rear wheel top
  27,  -32,  -17, //  21 Right rear wheel front top
  27,  -47,  -14, //  22 Right rear wheel front middle
  27,  -62,  -17, //  23 Right rear wheel front bottom
  27,  -67,  -24, //  24 Right rear wheel bottom
  27,  -62,  -31, //  25 Right rear wheel rear bottom

  27,  -47,   15, //  26 Right front wheel rear middle
  27,  -32,   18, //  27 Right front wheel rear top
  27,  -27,   25, //  28 Right front wheel top
  27,  -32,   32, //  29 Right front wheel front top
  27,  -47,   35, //  30 Right front wheel front middle
  27,  -62,   32, //  31 Right front wheel front bottom
  27,  -67,   25, //  32 Right front wheel bottom
  27,  -62,   18, //  33 Right front wheel rear bottom

  -27,  -47,  -34, //  34 Left rear wheel rear middle
  -27,  -32,  -31, //  35 Left rear wheel rear top
  -27,  -27,  -24, //  36 Left rear wheel top
  -27,  -32,  -17, //  37 Left rear wheel front top
  -27,  -47,  -14, //  38 Left rear wheel front middle
  -27,  -62,  -17, //  39 Left rear wheel front bottom
  -27,  -67,  -24, //  40 Left rear wheel bottom
  -27,  -62,  -31, //  41 Left rear wheel rear bottom

  -27,  -47,   15, //  42 Left front wheel rear middle
  -27,  -32,   18, //  43 Left front wheel rear top
  -27,  -27,   25, //  44 Left front wheel top
  -27,  -32,   32, //  45 Left front wheel front top
  -27,  -47,   35, //  46 Left front wheel front middle
  -27,  -62,   32, //  47 Left front wheel front bottom
  -27,  -67,   25, //  48 Left front wheel bottom
  -27,  -62,   18, //  49 Left front wheel rear bottom

   23,  -14,  39, //  50 Right headlight top left
   15,  -14,  39, //  51 Right headlight top right
   15,  -28,  39, //  52 Right headlight bottom right
   23,  -28,  39, //  53 Right headlight bottom left

  -23,  -14,  39, //  54 Left headlight top right
  -15,  -14,  39, //  55 Left headlight top left
  -15,  -28,  39, //  56 Left headlight bottom left
  -23,  -28,  39, //  57 Left headlight bottom right

   15,  -42,  39, //  58 Grill bottom left
  -15,  -42,  39, //  59 Grill bottom right

  26,              // Number of faces in the polygon mesh.
                  // Each face is (NPoints, VID, U, V) + texture id.
  4, 14, 0, 0, 15, 0, 0, 16, 0, 0, 0, 0, 0, 0, // Shadow
  4, 6, 0, 0, 13, 0, 0, 9, 0, 0, 2, 0, 0, 0, // Front window
  4, 2, 0, 0, 9, 0, 0, 10, 0, 0, 3, 0, 0, 9, // Hood
  4, 3, 0, 0, 10, 0, 0, 11, 0, 0, 4, 0, 0, 2, // Front
  4, 5, 0, 0, 12, 0, 0, 13, 0, 0, 6, 0, 0, 9, // Roof
  4, 1, 0, 0, 8, 0, 0, 12, 0, 0, 5, 0, 0, 0, // Rear window
  4, 17, 0, 0, 7, 0, 0, 8, 0, 0, 1, 0, 0, 2, // Rear
  5, 17, 0, 0, 1, 0, 0, 2, 0, 0, 3, 0, 0, 4, 0, 0, 7, // Right side
  4, 1, 0, 0, 5, 0, 0, 6, 0, 0, 2, 0, 0, 8, // Right window
  5, 7, 0, 0, 11, 0, 0, 10, 0, 0, 9, 0, 0, 8, 0, 0, 7, // Left side
  4, 8, 0, 0, 9, 0, 0, 13, 0, 0, 12, 0, 0, 8, // Left window
  4, 18, 0, 0, 19, 0, 0, 20, 0, 0, 21, 0, 0, 0, // Right rear wheel top part
  4, 18, 0, 0, 21, 0, 0, 22, 0, 0, 25, 0, 0, 2, // Right rear wheel middle part
  4, 25, 0, 0, 22, 0, 0, 23, 0, 0, 24, 0, 0, 0, // Right rear wheel bottom part
  4, 27, 0, 0, 28, 0, 0, 29, 0, 0, 30, 0, 0, 0, // Right front wheel top part
  4, 27, 0, 0, 30, 0, 0, 31, 0, 0, 26, 0, 0, 2, // Right front wheel middle part
  4, 31, 0, 0, 32, 0, 0, 33, 0, 0, 26, 0, 0, 0, // Right front wheel bottom part
  4, 37, 0, 0, 36, 0, 0, 35, 0, 0, 34, 0, 0, 0, // Left rear wheel top part
  4, 37, 0, 0, 34, 0, 0, 41, 0, 0, 38, 0, 0, 2, // Left rear wheel middle part
  4, 38, 0, 0, 41, 0, 0, 40, 0, 0, 39, 0, 0, 0, // Left rear wheel bottom part
  4, 46, 0, 0, 45, 0, 0, 44, 0, 0, 43, 0, 0, 0, // Left front wheel top part
  4, 46, 0, 0, 43, 0, 0, 42, 0, 0, 47, 0, 0, 2, // Left front wheel middle part
  4, 47, 0, 0, 42, 0, 0, 49, 0, 0, 48, 0, 0, 0, // Left front wheel bottom part
  4, 50, 0, 0, 51, 0, 0, 52, 0, 0, 53, 0, 0, 9, // Right headlight
  4, 55, 0, 0, 54, 0, 0, 57, 0, 0, 56, 0, 0, 9, // Left headlight
  4, 51, 0, 0, 55, 0, 0, 59, 0, 0, 58, 0, 0, 0 // Grill
};

const s8 small_tree[] =
{
  1,              // Number of vertex lists (Animation Frames)
  19,              // Number of verices per list.
   8,  -27,   9, // 0 Collision boundaries
  -2,  -27,  -9, // 1 Middle trunk bottom left
  -2,   10,  -5, // 2 Middle trunk top left
   1,   10,  -5, // 3 Middle trunk top right
   3,  -27,  -9, // 4 Middle trunk bottom right

   7,  -27,   0, // 5 Right trunk bottom
   4,   10,   0, // 6 Right trunk top

   -8, -27,   5, // 7 Left trunk bottom
   -5,  10,   0, // 8 Left trunk top

  -23, -27,  -2, // 9 Shadow

  -14,  38,   0, // 10 Crown middle 1
   -6,  40, -14, // 11 Crown middle 2
    6,  35, -14, // 12 Crown middle 3
   14,  38,   0, // 13 Crown middle 4

   -7,  64,   0, // 14 Crown top 1
   -3,  64,  -7, // 15 Crown top 2
    3,  63,  -7, // 16 Crown top 3
    7,  68,   0, // 17 Crown top 4

    0,  70,   0, // 18 Crown top
  13,              // Number of faces in the polygon mesh.
                  // Each face is (NPoints, VID, U, V) + texture id.
  3, 9, 0, 0, 7, 0, 0, 1, 0, 0, 0, // Shadow
  4, 7, 0, 0, 8, 0, 0, 2, 0, 0, 1, 0, 0, 2, // Left trunk
  4, 1, 0, 0, 2, 0, 0, 3, 0, 0, 4, 0, 0, 7, // Middle trunk
  4, 4, 0, 0, 3, 0, 0, 6, 0, 0, 5, 0, 0, 9, // Right trunk
  4, 8, 0, 0, 10, 0, 0, 11, 0, 0, 2, 0, 0, 0, // Crown bottom left
  4, 2, 0, 0, 11, 0, 0, 12, 0, 0, 3, 0, 0, 2, // Crown bottom middle
  4, 3, 0, 0, 12, 0, 0, 13, 0, 0, 6, 0, 0, 7, // Crown bottom right
  4, 10, 0, 0, 14, 0, 0, 15, 0, 0, 11, 0, 0, 2, // Crown middle left
  4, 11, 0, 0, 15, 0, 0, 16, 0, 0, 12, 0, 0, 7, // Crown middle middle
  4, 12, 0, 0, 16, 0, 0, 17, 0, 0, 13, 0, 0, 9, // Crown middle right
  3, 14, 0, 0, 18, 0, 0, 15, 0, 0, 7, // Crown top left
  3, 15, 0, 0, 18, 0, 0, 16, 0, 0, 7, // Crown top middle
  3, 16, 0, 0, 18, 0, 0, 17, 0, 0, 7 // Crown top right
};

const s8 tree[] =
{
  1,              // Number of vertex lists (Animation Frames)
  13,              // Number of verices per list.
    8,  -70,   8, // 0 Collision boundaries
   -8,  -70,   0, // 1 Trunk left
    0,  -15,   0, // 2 Trunk top
    0,  -70,  -8, // 3 Trunk bottom
    8,  -70,   0, // 4 Trunk right

  -20,  -15,   0, // 5 Crown left
    0,  127,   0, // 6 Crown top
    0,  -15, -20, // 7 Crown bottom
   20,  -15,   0, // 8 Crown right

  -40,  -70,   0, // 9 Left shadow
    0,  -70,  12, // 10 Top shadow
   17,  -70,   0, // 11 Right shadow
    0,  -70, -12, // 12 Bottom shadow
  5,              // Number of faces in the polygon mesh.
                  // Each face is (NPoints, VID, U, V) + texture id.
  4, 9, 0, 0, 10, 0, 0, 11, 0, 0, 12, 0, 0, 0, // Shadow
  3, 1, 0, 0, 2, 0, 0, 3, 0, 0, 2, // Left trunk
  3, 3, 0, 0, 2, 0, 0, 4, 0, 0, 7, // Right trunk
  3, 5, 0, 0, 6, 0, 0, 7, 0, 0, 2, // Left crown
  3, 7, 0, 0, 6, 0, 0, 8, 0, 0, 9 // Right crown
};

const s8 large_tree[] =
{
  1,              // Number of vertex lists (Animation Frames)
  13,              // Number of verices per list.
   11, -127,  11, // 0 Collision boundaries
  -11, -127,   0, // 1 Trunk left
    0,  -56,   0, // 2 Trunk top
    0, -127, -11, // 3 Trunk bottom
   11, -127,   0, // 4 Trunk right

  -26,  -56,   0, // 5 Crown left
    0,  127,   0, // 6 Crown top
    0,  -56, -26, // 7 Crown bottom
   26,  -56,   0, // 8 Crown right

  -52,  -127,   0, // 9 Left shadow
    0,  -127,  16, // 10 Top shadow
   22,  -127,   0, // 11 Right shadow
    0,  -127, -16, // 12 Bottom shadow
  5,              // Number of faces in the polygon mesh.
                  // Each face is (NPoints, VID, U, V) + texture id.
  4, 9, 0, 0, 10, 0, 0, 11, 0, 0, 12, 0, 0, 0, // Shadow
  3, 1, 0, 0, 2, 0, 0, 3, 0, 0, 2, // Left trunk
  3, 3, 0, 0, 2, 0, 0, 4, 0, 0, 7, // Right trunk
  3, 5, 0, 0, 6, 0, 0, 7, 0, 0, 2, // Left crown
  3, 7, 0, 0, 6, 0, 0, 8, 0, 0, 9 // Right crown
};

const s8 fuel_dump[] =
{
  1,              // Number of vertex lists (Animation Frames)
  24,              // Number of verices per list.
   22,  -120,  22, // 0 Right back shadow
   30,  -120,   0, // 1 Right shadow
   22,  -120, -22, // 2 Right front shadow
    0,  -120, -30, // 3 Front shadow
  -22,  -120, -22, // 4 Left front shadow
  -30,  -120,   0, // 5 Left shadow
  -22,  -120,  22, // 6 Left back shadow
    0,  -120,  30, // 7 Back shadow

   18,   30,  18, // 8 Right back middle
   25,   30,   0, // 9 Right middle
   18,   30, -18, // 10 Right front middle
    0,   30, -25, // 11 Front middle
  -18,   30, -18, // 12 Left front middle
  -25,   30,   0, // 13 Left middle
  -18,   30,  18, // 14 Left back middle
    0,   30,  25, // 15 Back middle

   22,  120,  22, // 16 Right back top
   30,  120,   0, // 17 Right top
   22,  120, -22, // 18 Right front top
    0,  120, -30, // 19 Front top
  -22,  120, -22, // 20 Left front top
  -30,  120,   0, // 21 Left top
  -22,  120,  22, // 22 Left back top
    0,  120,  30, // 23 Back top

  10,              // Number of faces in the polygon mesh.
                  // Each face is (NPoints, VID, U, V) + texture id.
  8, 0, 0, 0, 1, 0, 0, 2, 0, 0, 3, 0, 0, 4, 0, 0, 5, 0, 0, 6, 0, 0, 7, 0, 0, 0, // Shadow
  8, 8, 0, 0, 15, 0, 0, 14, 0, 0, 13, 0, 0, 12, 0, 0, 11, 0, 0, 10, 0, 0, 9, 0, 0, 0, // Tank bottom
  2, 14, 0, 0, 6, 0, 0, 3,// Left back leg
  2, 8, 0, 0, 0, 0, 0, 3,// Right back leg
  2, 10, 0, 0, 2, 0, 0, 3,// Right front leg
  2, 12, 0, 0, 4, 0, 0, 3, // Left front leg
  4, 13, 0, 0, 21, 0, 0, 20, 0, 0, 12, 0, 0, 2, // Left body
  4, 12, 0, 0, 20, 0, 0, 19, 0, 0, 11, 0, 0, 2, // Left middle body
  4, 11, 0, 0, 19, 0, 0, 18, 0, 0, 10, 0, 0, 7, // Right middle body
  4, 10, 0, 0, 18, 0, 0, 17, 0, 0, 9, 0, 0, 9 // Right body
};

const s8 cross[] =
{
  1,              // Number of vertex lists (Animation Frames)
  8,              // Number of verices per list.
   127,  0,  127, // Vertex X, Y, Z. Range (-120..120)
   127,  0, -127,
  -127,  0, -127,
  -127,  0,  127,

   127, 10,  127,
   127, 10, -127,
  -127, 10, -127,
  -127, 10,  127,
  4,              // Number of faces in the polygon mesh.
                  // Each face is (NPoints, VID, U, V) + texture id.
  4, 3, 0, 0, 7, 0, 0, 5, 0, 0, 1, 0, 0, 0,
  4, 2, 0, 0, 6, 0, 0, 4, 0, 0, 0, 0, 0, 0,
  2, 3, 0, 0, 1, 0, 0, 3,
  2, 0, 0, 0, 2, 0, 0, 3
};

const s8 peasant_hut[] =
{
  1,              // Number of vertex lists (Animation Frames)
  18,              // Number of verices per list.
   32, -43,  32, // 0 Right back bottom
   32, -43, -32, // 1 Right front bottom
  -32, -43, -32, // 2 Left front bottom
  -32, -43,  32, // 3 Left back bottom
   32,  43,  32, // 4 Right back top
   32,  43, -32, // 5 Right front top
  -32,  43, -32, // 6 Left front top
  -32,  43,  32, // 7 Left back top

   16,  68,  16, // 8 Right back roof
   16,  68, -16, // 9 Right front roof
  -16,  68, -16, // 10 Left front roof
  -16,  68,  16, // 11 Left back roof
    0, 110,   0, // 12 Rooftop

   -7, -43, -32, // 13 Door bottom left
   -7,  -7, -32, // 14 Door top left
    0,   7, -32, // 15 Door top
    7,  -7, -32, // 16 Door top right
    7, -43, -32, // 17 Door bottom right
  10,              // Number of faces in the polygon mesh.
                  // Each face is (NPoints, VID, U, V) + texture id.
  5, 7, 0, 0, 11, 0, 0, 12, 0, 0, 10, 0, 0, 6, 0, 0, 2, // Left roof
  5, 6, 0, 0, 10, 0, 0, 12, 0, 0, 9, 0, 0, 5, 0, 0, 5, // Front roof
  5, 5, 0, 0, 9, 0, 0, 12, 0, 0, 8, 0, 0, 4, 0, 0, 7, // Right roof
  4, 2, 0, 0, 6, 0, 0, 5, 0, 0, 1, 0, 0, 9, // Front
  4, 1, 0, 0, 5, 0, 0, 4, 0, 0, 0, 0, 0, 8, // Right side
  4, 3, 0, 0, 7, 0, 0, 6, 0, 0, 2, 0, 0, 8, // Left side
  5, 13, 0, 0, 14, 0, 0, 15, 0, 0, 16, 0, 0, 17, 0, 0, 7, // Door
  2, 14, 0, 0, 15, 0, 0, 0, // Door top left contour
  2, 15, 0, 0, 16, 0, 0, 0, // Door top left contour
  2, 16, 0, 0, 17, 0, 0, 0 // Door top left contour
};

const s8 church[] =
{
  1,              // Number of vertex lists (Animation Frames)
  37,              // Number of verices per list.
   45, -127,  45, // 0 Right back bottom
   45, -127, -45, // 1 Right front bottom
  -45, -127, -45, // 2 Left front bottom
  -45, -127,  45, // 3 Left back bottom
   45,  -11,  45, // 4 Right back top
   45,  -11, -45, // 5 Right front top
  -45,  -11, -45, // 6 Left front top
  -45,  -11,  45, // 7 Left back top

  -16,  30,  16, // 8 Left back roof
   16,  30,  16, // 9 Right back roof
   16,  30, -16, // 10 Right front roof
  -16,  30, -16, // 11 Left front roof

  -16, -127, -45, // 12 Door border bottom left
  -16,  -69, -45, // 13 Door border middle left
   -8,  -53, -45, // 14 Door border top left
    8,  -53, -45, // 15 Door border top right
   16,  -69, -45, // 16 Door border middle right
   16, -127, -45, // 17 Door border bottom right

  -10, -127, -45, // 18 Door bottom left
  -10,  -74, -45, // 19 Door middle left
   -6,  -66, -45, // 20 Door top left
    6,  -66, -45, // 21 Door top right
   10,  -74, -45, // 22 Door middle right
   10, -127, -45, // 23 Door bottom right

  -16,  100,  16, // 24 Left back tower
   16,  100,  16, // 25 Right back tower
   16,  100, -16, // 26 Right front tower
  -16,  100, -16, // 27 Left front tower

  -21,  100,  21, // 28 Left back tower roof
   21,  100,  21, // 29 Right back tower roof
   21,  100, -21, // 30 Right front tower roof
  -21,  100, -21, // 31 Left front tower roof
    0,  127,   0, // 32 Rooftop

    0,   86, -16, // 33 Cross top
    0,   55, -16, // 34 Cross bottom
   -5,   76, -16, // 35 Cross left
    5,   76, -16, // 36 Cross right
  17,              // Number of faces in the polygon mesh.
                  // Each face is (NPoints, VID, U, V) + texture id.
  4, 2, 0, 0, 6, 0, 0, 5, 0, 0, 1, 0, 0, 9, // Front
  4, 1, 0, 0, 5, 0, 0, 4, 0, 0, 0, 0, 0, 8, // Right side
  4, 3, 0, 0, 7, 0, 0, 6, 0, 0, 2, 0, 0, 8, // Left side
  6, 12, 0, 0, 13, 0, 0, 14, 0, 0, 15, 0, 0, 16, 0, 0, 17, 0, 0, 0, // Door border
  6, 18, 0, 0, 19, 0, 0, 20, 0, 0, 21, 0, 0, 22, 0, 0, 23, 0, 0, 7, // Door
  4, 28, 0, 0, 31, 0, 0, 30, 0, 0, 29, 0, 0, 0, // Tower roof bottom
  4, 11, 0, 0, 27, 0, 0, 26, 0, 0, 10, 0, 0, 9, // Tower front
  4, 10, 0, 0, 26, 0, 0, 25, 0, 0, 9, 0, 0, 8, // Tower right side
  4, 8, 0, 0, 24, 0, 0, 27, 0, 0, 11, 0, 0, 8, // Tower left side
  4, 7, 0, 0, 8, 0, 0, 11, 0, 0, 6, 0, 0, 2, // Left roof
  4, 6, 0, 0, 11, 0, 0, 10, 0, 0, 5, 0, 0, 5, // Front roof
  4, 5, 0, 0, 10, 0, 0, 9, 0, 0, 4, 0, 0, 7, // Right roof
  3, 28, 0, 0, 32, 0, 0, 31, 0, 0, 2, // Left tower roof
  3, 31, 0, 0, 32, 0, 0, 30, 0, 0, 5, // Front tower roof
  3, 30, 0, 0, 32, 0, 0, 29, 0, 0, 7, // Right tower roof
  2, 33, 0, 0, 34, 0, 0, 0, // Vertical cross
  2, 35, 0, 0, 36, 0, 0, 0 // Horizontal cross
};

const s8 rock[] =
{
  1,              // Number of vertex lists (Animation Frames)
  6,              // Number of verices per list.
   23, -30,   5, // 0 Bottom right back
   12, -30, -12, // 1 Bottom right
  -15, -30, -10, // 2 Bottom left
  -23, -30,   5, // 3 Bottom left back

   -8,  20,   0, // 4 Top left
   11,  30,   5, // 5 Top right
  4,              // Number of faces in the polygon mesh.
                  // Each face is (NPoints, VID, U, V) + texture id.
  3, 3, 0, 0, 4, 0, 0, 2, 0, 0, 2, // Left side face
  3, 2, 0, 0, 4, 0, 0, 1, 0, 0, 7, // Left face
  3, 1, 0, 0, 4, 0, 0, 5, 0, 0, 5, // Right face
  3, 1, 0, 0, 5, 0, 0, 0, 0, 0, 7 // Right side face
};

const s8 menhir[] =
{
  1,              // Number of vertex lists (Animation Frames)
  13,              // Number of verices per list.
   40, -120,  15, // 0 Collision boundaries
  -40, -120,  15, // 1 Bottom left back
  -30, -120,  -5, // 2 Bottom left
   -8, -120, -20, // 3 Bottom middle
   30, -120, -10, // 4 Bottom right
   40, -120,   5, // 5 Bottom right back

  -25,   55,   0, // 6 Middle left
  -17,  -10,  -5, // 7 Middle low
    2,   50, -10, // 8 Middle center
   25,   55,   0, // 9 Middle right

  -10,  120,   0, // 10 Top left
   10,  120,   0, // 11 Top right

  -60, -120,  -5, // 12 Shadow
  7,              // Number of faces in the polygon mesh.
                  // Each face is (NPoints, VID, U, V) + texture id.
  3, 12, 0, 0, 1, 0, 0, 3, 0, 0, 0, // Shadow
  4, 1, 0, 0, 6, 0, 0, 7, 0, 0, 2, 0, 0, 2, // Left face dark
  4, 6, 0, 0, 10, 0, 0, 8, 0, 0, 7, 0, 0, 5, // Left face top
  4, 2, 0, 0, 7, 0, 0, 8, 0, 0, 3, 0, 0, 5, // Left face bottom
  3, 10, 0, 0, 11, 0, 0, 8, 0, 0, 7, // Middle face top
  4, 3, 0, 0, 8, 0, 0, 11, 0, 0, 4, 0, 0, 7, // Middle face bottom
  4, 4, 0, 0, 11, 0, 0, 9, 0, 0, 5, 0, 0, 5 // Right face
};

const s8 allied_flag[] =
{
  1,              // Number of vertex lists (Animation Frames)
  5,              // Number of verices per list.
    0, -76,   0, // 0 Pole bottom
    0,  76,   0, // 1 Pole top
   23,  76,   0, // 2 Flag top right
   23,  46,   0, // 3 Flag bottom right
    0,  46,   0, // 4 Flag bottom left
  4,              // Number of faces in the polygon mesh.
                  // Each face is (NPoints, VID, U, V) + texture id.
  3, 1, 0, 0, 2, 0, 0, 4, 0, 0, 0, // Upper flag
  3, 2, 0, 0, 3, 0, 0, 4, 0, 0, 7, // Lower flag
  2, 4, 0, 0, 2, 0, 0, 3, // Flag stripe
  2, 0, 0, 0, 1, 0, 0, 2 // Flag pole
};

const s8 enemy_flag[] =
{
  1,              // Number of vertex lists (Animation Frames)
  7,              // Number of verices per list.
    0, -76,   0, // 0 Pole bottom
    0,  76,   0, // 1 Pole top
   23,  76,   0, // 2 Flag top right
   23,  46,   0, // 3 Flag bottom right
    0,  46,   0, // 4 Flag bottom left
    8,  61,   0, // 5 Flag middle left
   15,  61,   0, // 6 Flag middle right
  5,              // Number of faces in the polygon mesh.
                  // Each face is (NPoints, VID, U, V) + texture id.
  4, 1, 0, 0, 2, 0, 0, 3, 0, 0, 4, 0, 0, 0, // Flag black
  3, 1, 0, 0, 5, 0, 0, 4, 0, 0, 9, // Flag white
  3, 1, 0, 0, 6, 0, 0, 5, 0, 0, 7, // Flag light
  3, 4, 0, 0, 5, 0, 0, 6, 0, 0, 2, // Flag dark
  2, 0, 0, 0, 1, 0, 0, 2 // Flag pole
};

const s8 tent[] =
{
  1,              // Number of vertex lists (Animation Frames)
  16,              // Number of verices per list.
   25, -60,  25, // 0 Right back bottom
   27, -20,  25, // 1 Right back middle
    0,  60,  25, // 2 Back top
  -27, -20,  25, // 3 Left back middle
  -25, -60,  25, // 4 Left back bottom
  -35, -60,  28, // 5 Left back rope
   35, -60,  28, // 6 Right back rope

   25, -60, -25, // 7 Right front bottom
   27, -20, -25, // 8 Right front middle
    0,  60, -25, // 9 Front top
  -27, -20, -25, // 10 Left front middle
  -25, -60, -25, // 11 Left front bottom
   -9, -48, -25, // 12 Left zipper
    9, -48, -25, // 13 Right zipper
  -35, -60, -28, // 14 Left front rope
   35, -60, -28, // 15 Right front rope
  12,              // Number of faces in the polygon mesh.
                  // Each face is (NPoints, VID, U, V) + texture id.
  2, 5, 0, 0, 3, 0, 0, 3, // Left back rope
  2, 1, 0, 0, 6, 0, 0, 3, // Right back rope
  4, 4, 0, 0, 3, 0, 0, 10, 0, 0, 11, 0, 0, 0, // Left lower side
  4, 3, 0, 0, 2, 0, 0, 9, 0, 0, 10, 0, 0, 7, // Left upper side
  4, 7, 0, 0, 8, 0, 0, 1, 0, 0, 0, 0, 0, 7, // Right lower side
  4, 8, 0, 0, 9, 0, 0, 2, 0, 0, 1, 0, 0, 9, // Right upper side
  4, 11, 0, 0, 10, 0, 0, 9, 0, 0, 12, 0, 0, 2, // Left front
  4, 9, 0, 0, 8, 0, 0, 7, 0, 0, 13, 0, 0, 2, // Right front
  3, 12, 0, 0, 9, 0, 0, 13, 0, 0, 0, // Upper door
  4, 11, 0, 0, 12, 0, 0, 13, 0, 0, 7, 0, 0, 0, // Lower door
  2, 14, 0, 0, 10, 0, 0, 3, // Left front rope
  2, 8, 0, 0, 15, 0, 0, 3 // Right front rope
};

const s8 monument[] =
{
  1,              // Number of vertex lists (Animation Frames)
  25,              // Number of verices per list.
  -25, -127,  25, // 0 Base left back
   25, -127,  25, // 1 Base right back
   25, -127, -25, // 2 Base right front
  -25, -127, -25, // 3 Base left front

  -25, -105,  25, // 4 Hip left back
   25, -105,  25, // 5 Hip right back
   25, -105, -25, // 6 Hip right front
  -25, -105, -25, // 7 Hip left front

  -22,  -83,  22, // 8 Waist left back
   22,  -83,  22, // 9 Waist right back
   22,  -83, -22, // 10 Waist right front
  -22,  -83, -22, // 11 Waist left front

  -22,  -33,  22, // 12 Shoulder left back
   22,  -33,  22, // 13 Shoulder right back
   22,  -33, -22, // 14 Shoulder right front
  -22,  -33, -22, // 15 Shoulder left front

  -12,  -33,  12, // 16 Tower bottom left back
   12,  -33,  12, // 17 Tower bottom right back
   12,  -33, -12, // 18 Tower bottom right front
  -12,  -33, -12, // 19 Tower bottom left front

  -12,  100,  12, // 20 Tower top left back
   12,  100,  12, // 21 Tower top right back
   12,  100, -12, // 22 Tower top right front
  -12,  100, -12, // 23 Tower top left front

    0,  127,   0, // 24 Top

  16,              // Number of faces in the polygon mesh.
                  // Each face is (NPoints, VID, U, V) + texture id.
  4, 0, 0, 0, 4, 0, 0, 7, 0, 0, 3, 0, 0, 2, // Left bottom
  4, 3, 0, 0, 7, 0, 0, 6, 0, 0, 2, 0, 0, 7, // Front bottom
  4, 2, 0, 0, 6, 0, 0, 5, 0, 0, 1, 0, 0, 2, // Right bottom
  4, 4, 0, 0, 8, 0, 0, 11, 0, 0, 7, 0, 0, 8, // Left waist
  4, 7, 0, 0, 11, 0, 0, 10, 0, 0, 6, 0, 0, 9, // Front waist
  4, 6, 0, 0, 10, 0, 0, 9, 0, 0, 5, 0, 0, 8, // Right waist
  4, 8, 0, 0, 12, 0, 0, 15, 0, 0, 11, 0, 0, 7, // Left body
  4, 11, 0, 0, 15, 0, 0, 14, 0, 0, 10, 0, 0, 8, // Front body
  4, 10, 0, 0, 14, 0, 0, 13, 0, 0, 9, 0, 0, 7, // Right body
  4, 12, 0, 0, 13, 0, 0, 14, 0, 0, 15, 0, 0, 2, // Base top
  4, 16, 0, 0, 20, 0, 0, 23, 0, 0, 19, 0, 0, 8, // Left tower
  4, 19, 0, 0, 23, 0, 0, 22, 0, 0, 18, 0, 0, 9, // Front tower
  4, 18, 0, 0, 22, 0, 0, 21, 0, 0, 17, 0, 0, 8, // Right tower
  3, 20, 0, 0, 24, 0, 0, 23, 0, 0, 7, // Left top
  3, 23, 0, 0, 24, 0, 0, 22, 0, 0, 8, // Front top
  3, 22, 0, 0, 24, 0, 0, 21, 0, 0, 7 // Right top
};

const s8 wind_turbine[] =
{
  1,              // Number of vertex lists (Animation Frames)
  22,              // Number of verices per list.
    8, -127,   6, // 0 Base right
    0, -127, -11, // 1 Base front
   -8, -127,   6, // 2 Base left

    6,  -70,   4, // 3 Pole bottom right
    0,  -70,  -8, // 4 Pole bottom front
   -6,  -70,   4, // 5 Pole bottom left

    6,   17,   4, // 6 Pole top right
    0,   17,  -8, // 7 Pole top front
   -6,   17,   4, // 8 Pole top left

   -6,   17,  10, // 9 Box bottom left back
    6,   17,  10, // 10 Box bottom right back
    6,   17, -17, // 11 Box bottom right front
   -6,   17, -17, // 12 Box bottom left front

   -6,   37,  10, // 13 Box top left back
    6,   37,  10, // 14 Box top right back
    6,   37, -17, // 15 Box top right front
   -6,   37, -17, // 16 Box top left front

   36,   91, -17, // 17 Top blade tip
   22,   50, -17, // 18 Top blade middle
    0,   27, -17, // 19 Propeller center
  -22,    4, -17, // 20 Bottom blade middle
  -36,  -37, -17, // 21 Bottom blade tip

  11,              // Number of faces in the polygon mesh.
                  // Each face is (NPoints, VID, U, V) + texture id.
  4, 9, 0, 0, 12, 0, 0, 11, 0, 0, 10, 0, 0, 0, // Box bottom
  4, 2, 0, 0, 5, 0, 0, 4, 0, 0, 1, 0, 0, 2, // Left base
  4, 1, 0, 0, 4, 0, 0, 3, 0, 0, 0, 0, 0, 7, // Right base
  4, 5, 0, 0, 8, 0, 0, 7, 0, 0, 4, 0, 0, 2, // Left pole
  4, 4, 0, 0, 7, 0, 0, 6, 0, 0, 3, 0, 0, 7, // Right pole
  4, 9, 0, 0, 13, 0, 0, 16, 0, 0, 12, 0, 0, 2, // Box left
  4, 12, 0, 0, 16, 0, 0, 15, 0, 0, 11, 0, 0, 7, // Box front
  4, 11, 0, 0, 15, 0, 0, 14, 0, 0, 10, 0, 0, 8, // Box right
  4, 9, 0, 0, 10, 0, 0, 11, 0, 0, 12, 0, 0, 0, // Box top
  3, 17, 0, 0, 18, 0, 0, 19, 0, 0, 9, // Top blade
  3, 19, 0, 0, 21, 0, 0, 20, 0, 0, 9 // Bottom blade
};

const s8 yellow_flower[] =
{
  1,              // Number of vertex lists (Animation Frames)
  14,              // Number of verices per list.
 -127,   0,-127, // 0 Collision boundaries
    0,   0,   0, // 1 Root
    0,  34,   0, // 2 Stalk
   -3,  51,   0, // 3 Left flower
    3,  51,   0, // 4 Right flower
    0,  57,   0, // 5 Top

    3,  19,   0, // 6 Right leaf left
   11,  28,   0, // 7 Right leaf top
    8,   6,   0, // 8 Right leaf bottom

   -9,   9,   0, // 9 Left leaf bottom
  -11,  26,   0, // 10 Left leaf top
   -4,  19,   0, // 11 Left leaf right

  -17,   0,   3, // 12 Shadow back
  -13,   0,  -3, // 13 Shadow front

  6,              // Number of faces in the polygon mesh.
                  // Each face is (NPoints, VID, U, V) + texture id.
  3, 1, 0, 0, 13, 0, 0, 12, 0, 0, 0, // Shadow
  2, 2, 0, 0, 5, 0, 0, 3, // Top
  2, 1, 0, 0, 2, 0, 0, 2, // Stalk
  3, 2, 0, 0, 3, 0, 0, 4, 0, 0, 8, // Flower
  4, 1, 0, 0, 9, 0, 0, 10, 0, 0, 11, 0, 0, 2, // Left leaf
  4, 1, 0, 0, 6, 0, 0, 7, 0, 0, 8, 0, 0, 7 // Right leaf
};

const s8 red_flower[] =
{
  1,              // Number of vertex lists (Animation Frames)
  17,              // Number of verices per list.
 -127,   0,-127, // 0 Collision boundaries
    0,   0,   0, // 1 Root
    4,  60,   0, // 2 Stalk

    0,   0,  -2, // 3 Right leaf left
   10,  36,  -5, // 4 Right leaf top
    7,   0,   2, // 5 Right leaf right

   15,  25,   2, // 6 Rightmost leaf top
   10,   0,   5, // 7 Rightmost leaf right

   -3,   0,   2, // 8 Left leaf left
   -5,  44,  -3, // 9 Left leaf top

   -5,   0,   5, // 10 Leftmost leaf left
  -13,  31,   2, // 11 Leftmost leaf top

    6,  60,  -2, // 12 Pistil

    0,  60,  -3, // 13 Petal left
    4,  66,  -3, // 14 Petal top
    8,  60,  -3, // 15 Petal right
    4,  54,  -3, // 16 Petal bottom
  10,              // Number of faces in the polygon mesh.
                  // Each face is (NPoints, VID, U, V) + texture id.
  3, 8, 0, 0, 5, 0, 0, 3, 0, 0, 0, // Shadow
  2, 1, 0, 0, 2, 0, 0, 3, // Stalk
  4, 13, 0, 0, 14, 0, 0, 15, 0, 0, 16, 0, 0, 2, // Petals
  2, 2, 0, 0, 12, 0, 0, 3, // Pistil
  3, 10, 0, 0, 11, 0, 0, 8, 0, 0, 0, // Leftmost leaf
  3, 8, 0, 0, 11, 0, 0, 10, 0, 0, 0, // Leftmost leaf back
  3, 8, 0, 0, 9, 0, 0, 3, 0, 0, 2, // Left leaf
  3, 3, 0, 0, 9, 0, 0, 8, 0, 0, 2, // Left leaf back
  3, 3, 0, 0, 4, 0, 0, 5, 0, 0, 2, // Right leaf
  3, 5, 0, 0, 6, 0, 0, 7, 0, 0, 7 // Rightmost leaf
};

const s8 shed[] =
{
  1,              // Number of vertex lists (Animation Frames)
  18,              // Number of verices per list.
  -48, -57,  70, // 0 Left back bottom
   48, -57,  70, // 1 Right back bottom
   48, -57, -70, // 2 Right front bottom
  -48, -57, -70, // 3 Left front bottom

  -48,  41,  70, // 4 Left back top
   48,  41,  70, // 5 Right back top
   48,  41, -70, // 6 Right front top
  -48,  41, -70, // 7 Left front top

    0,  86,  55, // 8 Back rooftop
    0,  86, -55, // 9 Front rooftop

   12, -57, -70, // 10 Door bottom left
   12,  21, -70, // 11 Door top left
   33,  21, -70, // 12 Door top right
   33, -57, -70, // 13 Door bottom right

  -33, -21, -70, // 14 Window bottom left
  -33,  21, -70, // 15 Window top left
   -9,  21, -70, // 16 Window top right
   -9, -21, -70, // 17 Window bottom right

  8,              // Number of faces in the polygon mesh.
                  // Each face is (NPoints, VID, U, V) + texture id.
  4, 4, 0, 0, 8, 0, 0, 9, 0, 0, 7, 0, 0, 3, // Left roof
  3, 7, 0, 0, 9, 0, 0, 6, 0, 0, 7, // Front roof
  4, 6, 0, 0, 9, 0, 0, 8, 0, 0, 5, 0, 0, 9, // Right roof
  4, 3, 0, 0, 7, 0, 0, 6, 0, 0, 2, 0, 0, 2, // Front
  4, 2, 0, 0, 6, 0, 0, 5, 0, 0, 1, 0, 0, 7, // Right side
  4, 0, 0, 0, 4, 0, 0, 7, 0, 0, 3, 0, 0, 1, // Left side
  4, 10, 0, 0, 11, 0, 0, 12, 0, 0, 13, 0, 0, 0, // Door
  4, 14, 0, 0, 15, 0, 0, 16, 0, 0, 17, 0, 0, 0 // Window
};

const s8 out_post[] =
{
  1,              // Number of vertex lists (Animation Frames)
  20,              // Number of verices per list.
  -50, -57,  50, // 0 Left back bottom
   50, -57,  50, // 1 Right back bottom
   50, -57, -50, // 2 Right front bottom
  -50, -57, -50, // 3 Left front bottom

  -50,  53,  50, // 4 Left back top
   50,  53,  50, // 5 Right back top
   50,  53, -50, // 6 Right front top
  -50,  53, -50, // 7 Left front top

  -50,  69,  50, // 8 Left back roof
   50,  69,  50, // 9 Right back roof
   50,  69, -50, // 10 Right front roof
  -50,  69, -50, // 11 Left front roof

   14, -57, -50, // 12 Door bottom left
   14,  30, -50, // 13 Door top left
   40,  30, -50, // 14 Door top right
   40, -57, -50, // 15 Door bottom right

  -38, -31, -50, // 16 Window bottom left
  -38,  30, -50, // 17 Window top left
    7,  30, -50, // 18 Window top right
    7, -31, -50, // 19 Window bottom right

  11,              // Number of faces in the polygon mesh.
                  // Each face is (NPoints, VID, U, V) + texture id.
  4, 4, 0, 0, 8, 0, 0, 11, 0, 0, 7, 0, 0, 3, // Left roof
  4, 7, 0, 0, 11, 0, 0, 10, 0, 0, 6, 0, 0, 7, // Front roof
  4, 6, 0, 0, 10, 0, 0, 9, 0, 0, 5, 0, 0, 8, // Right roof
  4, 8, 0, 0, 9, 0, 0, 10, 0, 0, 11, 0, 0, 9, // Top roof
  4, 3, 0, 0, 7, 0, 0, 6, 0, 0, 2, 0, 0, 2, // Front
  4, 2, 0, 0, 6, 0, 0, 5, 0, 0, 1, 0, 0, 7, // Right side
  4, 0, 0, 0, 4, 0, 0, 7, 0, 0, 3, 0, 0, 1, // Left side
  4, 12, 0, 0, 13, 0, 0, 14, 0, 0, 15, 0, 0, 0, // Door
  4, 16, 0, 0, 17, 0, 0, 18, 0, 0, 19, 0, 0, 0, // Window
  2, 16, 0, 0, 17, 0, 0, 2, // Window left border
  2, 16, 0, 0, 19, 0, 0, 2 // Window bottom border
};

const s8 house[] =
{
  1,              // Number of vertex lists (Animation Frames)
  23,              // Number of verices per list.
   82, -90,  82, // 0 Right back bottom
  -82, -90,  82, // 1 Left back bottom
   82, -90, -82, // 2 Right front bottom
  -82, -90, -82, // 3 Left front bottom

  -82,  43,  82, // 4 Left back top
   82,  43,  82, // 5 Right back top
   82,  43, -82, // 6 Right front top
  -82,  43, -82, // 7 Left front top

    0, 127,   0, // 8 Rooftop

  -67, -90, -82, // 9 Door bottom left
  -67,   8, -82, // 10 Door top left
  -39,   8, -82, // 11 Door top right
  -39, -90, -82, // 12 Door bottom right

   -2, -60, -82, // 13 Front window bottom left
   -2,  17, -82, // 14 Front window top left
   67,  17, -82, // 15 Front window top right
   67, -60, -82, // 16 Front window bottom right

   14,  17, -82, // 17 Left curtain
   50,  17, -82, // 18 Right curtain

  -82, -60,  67, // 19 Side window back bottom
  -82,  17,  67, // 20 Side window back top
  -82,  17,   2, // 21 Side window front top
  -82, -60,   2, // 22 Side window front bottom

  11,              // Number of faces in the polygon mesh.
                  // Each face is (NPoints, VID, U, V) + texture id.
  3, 4, 0, 0, 8, 0, 0, 7, 0, 0, 2, // Left roof
  3, 7, 0, 0, 8, 0, 0, 6, 0, 0, 5, // Front roof
  3, 6, 0, 0, 8, 0, 0, 5, 0, 0, 7, // Right roof
  4, 3, 0, 0, 7, 0, 0, 6, 0, 0, 2, 0, 0, 9, // Front
  4, 2, 0, 0, 6, 0, 0, 5, 0, 0, 0, 0, 0, 8, // Right side
  4, 1, 0, 0, 4, 0, 0, 7, 0, 0, 3, 0, 0, 8, // Left side
  4, 9, 0, 0, 10, 0, 0, 11, 0, 0, 12, 0, 0, 2, // Door
  4, 13, 0, 0, 14, 0, 0, 15, 0, 0, 16, 0, 0, 0, // Front window
  3, 13, 0, 0, 14, 0, 0, 17, 0, 0, 2, // Left curtain
  3, 18, 0, 0, 15, 0, 0, 16, 0, 0, 2, // Right curtain
  4, 19, 0, 0, 20, 0, 0, 21, 0, 0, 22, 0, 0, 0 // Side window
};

const s8 bungalow[] =
{
  1,              // Number of vertex lists (Animation Frames)
  28,              // Number of verices per list.
   67, -80,  50, // 0 Right back bottom
   67, -80, -50, // 1 Right front bottom
  -67, -80, -50, // 2 Left front bottom
  -67, -80,  50, // 3 Left back bottom

  -67,  43,  50, // 4 Left back top
   67,  43,  50, // 5 Right back top
   67,  43, -50, // 6 Right front top
  -67,  43, -50, // 7 Left front top

  -52,  75,  35, // 8 Left back roof
   52,  75,  35, // 9 Right back roof
   52,  75, -35, // 10 Right front roof
  -52,  75, -35, // 11 Left front roof

  -26, 100,   0, // 12 Left rooftop
   26, 100,   0, // 13 Right rooftop

   26, -80, -50, // 14 Door bottom left
   26,   0, -50, // 15 Door top left
   52,   0, -50, // 16 Door top right
   52, -80, -50, // 17 Door bottom right

  -52, -42, -50, // 18 Window bottom left
  -52,  29, -50, // 19 Window top left
    3,  29, -50, // 20 Window top right
    3, -42, -50, // 21 Window bottom right

  -52, -32, -50, // 22 Window border left
    3, -32, -50, // 23 Window border right

  -28,  29, -50, // 24 Left curtain top
  -35, -32, -50, // 25 Left curtain bottom
  -21,  29, -50, // 26 Right curtain top
  -14, -32, -50, // 27 Right curtain bottom

  14,              // Number of faces in the polygon mesh.
                  // Each face is (NPoints, VID, U, V) + texture id.
  4, 4, 0, 0, 8, 0, 0, 11, 0, 0, 7, 0, 0, 2, // Left lower roof
  4, 7, 0, 0, 11, 0, 0, 10, 0, 0, 6, 0, 0, 5, // Front lower roof
  4, 6, 0, 0, 10, 0, 0, 9, 0, 0, 5, 0, 0, 2, // Right lower roof
  3, 8, 0, 0, 12, 0, 0, 11, 0, 0, 5, // Left upper roof
  4, 11, 0, 0, 12, 0, 0, 13, 0, 0, 10, 0, 0, 7, // Front upper roof
  3, 10, 0, 0, 13, 0, 0, 9, 0, 0, 5, // Right upper roof
  4, 2, 0, 0, 7, 0, 0, 6, 0, 0, 1, 0, 0, 9, // Front
  4, 1, 0, 0, 6, 0, 0, 5, 0, 0, 0, 0, 0, 8, // Right side
  4, 3, 0, 0, 4, 0, 0, 7, 0, 0, 2, 0, 0, 8, // Left side
  4, 14, 0, 0, 15, 0, 0, 16, 0, 0, 17, 0, 0, 2, // Door
  4, 22, 0, 0, 19, 0, 0, 20, 0, 0, 23, 0, 0, 0, // Window
  4, 22, 0, 0, 19, 0, 0, 24, 0, 0, 25, 0, 0, 7, // Left curtain
  4, 27, 0, 0, 26, 0, 0, 20, 0, 0, 23, 0, 0, 7, // Right curtain
  4, 18, 0, 0, 22, 0, 0, 23, 0, 0, 21, 0, 0, 2 // Window border
};

const s8 hospital[] =
{
  1,              // Number of vertex lists (Animation Frames)
  34,              // Number of verices per list.
  -64, -80,  64, // 0 Left back bottom
   64, -80,  64, // 1 Right back bottom
   64, -80, -64, // 2 Right front bottom
  -64, -80, -64, // 3 Left front bottom

  -64,  46,  64, // 4 Left back top
   64,  46,  64, // 5 Right back top
   64,  46, -64, // 6 Right front top
  -64,  46, -64, // 7 Left front top

  -21, 116,  64, // 8 Left back roof
   21, 116,  64, // 9 Right back roof
   21, 116, -64, // 10 Right front roof
  -21, 116, -64, // 11 Left front roof

  -21, -80, -64, // 12 Door bottom left
  -21,   0, -64, // 13 Door top left
   21,   0, -64, // 14 Door top right
   21, -80, -64, // 15 Door bottom right

  -21,  46, -64, // 16 Sign bottom left
   21,  46, -64, // 17 Sign bottom right

   -5,  55, -64, // 18 Cross vertical bottom left
   -5, 107, -64, // 19 Cross vertical top left
    5, 107, -64, // 20 Cross vertical top right
    5,  55, -64, // 21 Cross vertical bottom right

  -15,  72, -64, // 22 Cross horizontal bottom left
  -15,  90, -64, // 23 Cross horizontal top left
   15,  90, -64, // 24 Cross horizontal top right
   15,  72, -64, // 25 Cross horizontal bottom right

  -64, -44,  44, // 26 Left window back bottom
  -64,   0,  44, // 27 Left window back top
  -64,   0, -44, // 28 Left window front top
  -64, -44, -44, // 29 Left window front bottom

   64, -44,  44, // 30 Right window back bottom
   64,   0,  44, // 31 Right window back top
   64,   0, -44, // 32 Right window front top
   64, -44, -44, // 33 Right window front bottom

  16,              // Number of faces in the polygon mesh.
                  // Each face is (NPoints, VID, U, V) + texture id.
  4, 0, 0, 0, 4, 0, 0, 7, 0, 0, 3, 0, 0, 1, // Left side
  4, 26, 0, 0, 27, 0, 0, 28, 0, 0, 29, 0, 0, 0, // Left window
  2, 26, 0, 0, 29, 0, 0, 3, // Left window border
  4, 2, 0, 0, 6, 0, 0, 5, 0, 0, 1, 0, 0, 7, // Right side
  4, 33, 0, 0, 32, 0, 0, 31, 0, 0, 30, 0, 0, 0, // Right window
  2, 30, 0, 0, 33, 0, 0, 3, // Right window border
  4, 3, 0, 0, 7, 0, 0, 6, 0, 0, 2, 0, 0, 2, // Front
  4, 12, 0, 0, 13, 0, 0, 14, 0, 0, 15, 0, 0, 0, // Door
  4, 4, 0, 0, 8, 0, 0, 11, 0, 0, 7, 0, 0, 3, // Left roof
  4, 8, 0, 0, 9, 0, 0, 10, 0, 0, 11, 0, 0, 7, // Top roof
  4, 6, 0, 0, 10, 0, 0, 9, 0, 0, 5, 0, 0, 9, // Right roof
  3, 7, 0, 0, 11, 0, 0, 16, 0, 0, 3, // Left front roof
  4, 16, 0, 0, 11, 0, 0, 10, 0, 0, 17, 0, 0, 9, // Sign
  3, 10, 0, 0, 6, 0, 0, 17, 0, 0, 3, // Right front roof
  4, 18, 0, 0, 19, 0, 0, 20, 0, 0, 21, 0, 0, 2, // Cross vertical
  4, 22, 0, 0, 23, 0, 0, 24, 0, 0, 25, 0, 0, 2 // Cross horizontal
};

const s8 hangar[] =
{
  1,              // Number of vertex lists (Animation Frames)
  18,              // Number of verices per list.
   110, -120,  115, // 0 Right back bottom
   110, -120, -115, // 1 Right front bottom
  -110, -120, -115, // 2 Left front bottom
  -110, -120,  115, // 3 Left back bottom

   110,   56,  115, // 4 Right back top
   110,   56, -115, // 5 Right front top
  -110,   56, -115, // 6 Left front top
  -110,   56,  115, // 7 Left back top

    55,  104,  115, // 8 Right back roof
    55,  104, -115, // 9 Right front roof
   -55,  104, -115, // 10 Left front roof
   -55,  104,  115, // 11 Left back roof

     0,  120,  115, // 12 Back rooftop
     0,  120, -115, // 13 Front rooftop

   -76, -120, -115, // 14 Door bottom left
   -76,   41, -115, // 15 Door top left
    76,   41, -115, // 16 Door top right
    76, -120, -115, // 17 Door bottom right

  8,              // Number of faces in the polygon mesh.
                  // Each face is (NPoints, VID, U, V) + texture id.
  4, 7, 0, 0, 11, 0, 0, 10, 0, 0, 6, 0, 0, 3, // Leftmost roof
  4, 11, 0, 0, 12, 0, 0, 13, 0, 0, 10, 0, 0, 7, // Left roof
  4, 12, 0, 0, 8, 0, 0, 9, 0, 0, 13, 0, 0, 9, // Right roof
  4, 8, 0, 0, 4, 0, 0, 5, 0, 0, 9, 0, 0, 8, // Rightmost roof
  7, 2, 0, 0, 6, 0, 0, 10, 0, 0, 13, 0, 0, 9, 0, 0, 5, 0, 0, 1, 0, 0, 2, // Front
  4, 1, 0, 0, 5, 0, 0, 4, 0, 0, 0, 0, 0, 7, // Right side
  4, 3, 0, 0, 7, 0, 0, 6, 0, 0, 2, 0, 0, 1, // Left side
  4, 14, 0, 0, 15, 0, 0, 16, 0, 0, 17, 0, 0, 0 // Door
};

const s8 player[] =
{
1,
59,
 9, -44,  10,
 2,  25,  2,
-2,  25,  2,
-3,  31,  3,
 3,  31,  3,
-7, -15,  0,
-9,   5, -3,
-7,   4,  1,
-6, -12,  1,
 7,  -6,  3,
 7,   4,  1,
 9,   1, -3,
 6,  -3,  5,
 1, -21,  3,
 5, -21,  3,
 4, -40,  2,
 1, -40,  2,
-5,   0, -2,
 0,  -6, -2,
 5,   0, -2,
 2,  38, -2,
 2,  38,  2,
-2,  38,  2,
-2,  38, -2,
-3,  31, -3,
 3,  31, -3,
 5,   0,  3,
-5,   0,  3,
 5,  20,  3,
-5,  20,  3,
-7,  22, -3,
 7,  22, -3,
-5, -21, -1,
-1, -21, -1,
 5, -21, -1,
 2,  26, -2,
-2,  26, -2,
 0,  -6,  3,
-5, -21,  3,
 1, -21, -1,
-1, -21,  3,
 1, -40, -2,
 4, -40, -2,
-1, -40,  2,
-4, -40,  2,
-4, -40, -2,
-1, -40, -2,
 6,  -8,  3,
 6,  -8,  5,
 6,  -3,  5,
 6,  -3,  3,
 4, -44, -2,
 4, -44,  6,
-1, -44, -2,
-1, -44,  6,
 1, -44,  6,
 1, -44, -2,
-4, -44, -2,
-4, -44,  6,
94,
3, 42, 0, 0, 15, 0, 0, 51, 0, 0, 0, // Shoe
3, 52, 0, 0, 51, 0, 0, 15, 0, 0, 0, // Shoe
3, 46, 0, 0, 43, 0, 0, 53, 0, 0, 0, // Shoe
3, 54, 0, 0, 53, 0, 0, 43, 0, 0, 0, // Shoe
3, 16, 0, 0, 41, 0, 0, 55, 0, 0, 0, // Shoe
3, 56, 0, 0, 55, 0, 0, 41, 0, 0, 0, // Shoe
3, 45, 0, 0, 46, 0, 0, 57, 0, 0, 0, // Shoe
3, 53, 0, 0, 57, 0, 0, 46, 0, 0, 0, // Shoe
3, 15, 0, 0, 16, 0, 0, 52, 0, 0, 0, // Shoe
3, 55, 0, 0, 52, 0, 0, 16, 0, 0, 0, // Shoe
3, 41, 0, 0, 42, 0, 0, 56, 0, 0, 0, // Shoe
3, 51, 0, 0, 56, 0, 0, 42, 0, 0, 0, // Shoe
3, 43, 0, 0, 44, 0, 0, 54, 0, 0, 0, // Shoe
3, 58, 0, 0, 54, 0, 0, 44, 0, 0, 0, // Shoe
3, 44, 0, 0, 45, 0, 0, 58, 0, 0, 0, // Shoe
3, 57, 0, 0, 58, 0, 0, 45, 0, 0, 0, // Shoe
3, 33, 0, 0, 40, 0, 0, 46, 0, 0, 0, // Left leg lower inside
3, 43, 0, 0, 46, 0, 0, 40, 0, 0, 0, // Left leg lower inside
3, 37, 0, 0, 33, 0, 0, 18, 0, 0, 0, // Left leg upper inside
3, 40, 0, 0, 33, 0, 0, 37, 0, 0, 0, // Left leg upper inside
3, 13, 0, 0, 39, 0, 0, 16, 0, 0, 0, // Right leg lower inside
3, 41, 0, 0, 16, 0, 0, 39, 0, 0, 0, // Right leg lower inside
3, 37, 0, 0, 39, 0, 0, 13, 0, 0, 0, // Right leg upper inside
3, 18, 0, 0, 39, 0, 0, 37, 0, 0, 0, // Right leg upper inside
3, 40, 0, 0, 38, 0, 0, 43, 0, 0, 7, // Left leg lower front
3, 44, 0, 0, 43, 0, 0, 38, 0, 0, 7, // Left leg lower front
3, 38, 0, 0, 40, 0, 0, 37, 0, 0, 7, // Left leg upper front
3, 27, 0, 0, 38, 0, 0, 37, 0, 0, 7, // Left leg upper front
3, 14, 0, 0, 13, 0, 0, 15, 0, 0, 2, // Right leg lower front
3, 16, 0, 0, 15, 0, 0, 13, 0, 0, 2, // Right leg lower front
3, 13, 0, 0, 14, 0, 0, 37, 0, 0, 7, // Right leg upper front
3, 37, 0, 0, 14, 0, 0, 26, 0, 0, 7, // Right leg upper front
3, 27, 0, 0, 37, 0, 0, 26, 0, 0, 7, // Crotch
3, 32, 0, 0, 33, 0, 0, 45, 0, 0, 0, // Left leg lower back
3, 46, 0, 0, 45, 0, 0, 33, 0, 0, 0, // Left leg lower back
3, 33, 0, 0, 32, 0, 0, 17, 0, 0, 2, // Left leg upper back
3, 18, 0, 0, 33, 0, 0, 17, 0, 0, 2, // Left leg upper back
3, 39, 0, 0, 34, 0, 0, 41, 0, 0, 7, // Right leg lower back
3, 42, 0, 0, 41, 0, 0, 34, 0, 0, 7, // Right leg lower back
3, 19, 0, 0, 39, 0, 0, 18, 0, 0, 7, // Right leg upper back
3, 34, 0, 0, 39, 0, 0, 19, 0, 0, 7, // Right leg upper back
3, 18, 0, 0, 17, 0, 0, 19, 0, 0, 2, // Ass
3, 38, 0, 0, 32, 0, 0, 44, 0, 0, 7, // Left leg lower outside
3, 45, 0, 0, 44, 0, 0, 32, 0, 0, 7, // Left leg lower outside
3, 17, 0, 0, 38, 0, 0, 27, 0, 0, 8, // Left leg upper outside
3, 32, 0, 0, 38, 0, 0, 17, 0, 0, 8, // Left leg upper outside
3, 34, 0, 0, 14, 0, 0, 42, 0, 0, 0, // Right leg lower outside
3, 15, 0, 0, 42, 0, 0, 14, 0, 0, 0, // Right leg lower outside
3, 19, 0, 0, 14, 0, 0, 34, 0, 0, 2, // Right leg upper outside
3, 26, 0, 0, 14, 0, 0, 19, 0, 0, 2, // Right leg upper outside
3, 27, 0, 0, 26, 0, 0, 28, 0, 0, 2, // Front torso
3, 29, 0, 0, 27, 0, 0, 28, 0, 0, 2, // Front torso
3, 17, 0, 0, 27, 0, 0, 29, 0, 0, 2, // Left torso
3, 17, 0, 0, 29, 0, 0, 30, 0, 0, 2, // Left torso
3, 26, 0, 0, 19, 0, 0, 31, 0, 0, 0, // Right torso
3, 26, 0, 0, 31, 0, 0, 28, 0, 0, 0, // Right torso
3, 19, 0, 0, 17, 0, 0, 30, 0, 0, 2, // Back torso
3, 31, 0, 0, 19, 0, 0, 30, 0, 0, 2, // Back torso
3, 35, 0, 0, 31, 0, 0, 36, 0, 0, 7, // Back shoulders
3, 30, 0, 0, 36, 0, 0, 31, 0, 0, 7, // Back shoulders
3, 29, 0, 0, 28, 0, 0, 1, 0, 0, 7, // Chest
3, 29, 0, 0, 1, 0, 0, 2, 0, 0, 7, // Chest
3, 28, 0, 0, 31, 0, 0, 35, 0, 0, 7, // Right shoulder
3, 28, 0, 0, 35, 0, 0, 1, 0, 0, 7, // Right shoulder
3, 36, 0, 0, 29, 0, 0, 2, 0, 0, 8, // Left shoulder
3, 29, 0, 0, 36, 0, 0, 30, 0, 0, 8, // Left shoulder
3, 30, 0, 0, 7, 0, 0, 29, 0, 0, 7, // Left upper arm
3, 6, 0, 0, 7, 0, 0, 30, 0, 0, 7, // Left upper arm
3, 6, 0, 0, 5, 0, 0, 7, 0, 0, 9, // Left lower arm
3, 5, 0, 0, 8, 0, 0, 7, 0, 0, 9, // Left lower arm
3, 48, 0, 0, 47, 0, 0, 49, 0, 0, 0, // Gun
3, 50, 0, 0, 49, 0, 0, 47, 0, 0, 0, // Gun
3, 28, 0, 0, 10, 0, 0, 31, 0, 0, 2, // Right upper arm
3, 11, 0, 0, 31, 0, 0, 10, 0, 0, 2, // Right upper arm
3, 10, 0, 0, 9, 0, 0, 11, 0, 0, 9, // Right lower arm
3, 12, 0, 0, 9, 0, 0, 10, 0, 0, 9, // Right lower arm
3, 35, 0, 0, 36, 0, 0, 25, 0, 0, 0, // Head back
3, 24, 0, 0, 25, 0, 0, 36, 0, 0, 0, // Head back
3, 3, 0, 0, 24, 0, 0, 2, 0, 0, 0, // Head left
3, 36, 0, 0, 2, 0, 0, 24, 0, 0, 0, // Head left
3, 1, 0, 0, 35, 0, 0, 4, 0, 0, 0, // Head right
3, 25, 0, 0, 4, 0, 0, 35, 0, 0, 0, // Head right
3, 2, 0, 0, 1, 0, 0, 3, 0, 0, 9, // Face
3, 4, 0, 0, 3, 0, 0, 1, 0, 0, 9, // Face
3, 22, 0, 0, 24, 0, 0, 3, 0, 0, 0, // Hat left
3, 23, 0, 0, 24, 0, 0, 22, 0, 0, 0, // Hat left
3, 20, 0, 0, 21, 0, 0, 25, 0, 0, 0, // Hat right
3, 4, 0, 0, 25, 0, 0, 21, 0, 0, 0, // Hat right
3, 23, 0, 0, 20, 0, 0, 24, 0, 0, 2, // Hat back
3, 25, 0, 0, 24, 0, 0, 20, 0, 0, 2, // Hat back
3, 21, 0, 0, 22, 0, 0, 4, 0, 0, 2, // Hat front
3, 3, 0, 0, 4, 0, 0, 22, 0, 0, 2, // Hat front
3, 21, 0, 0, 20, 0, 0, 22, 0, 0, 2, // Hat top
3, 23, 0, 0, 22, 0, 0, 20, 0, 0, 2 // Hat top
};

const s8 swimmer[] =
{
1,
32,
 15,  0, 12,
  2,  4,  4,
 13,  7,  5,
 11,  0,  6,
  5,  0, -9,
 -5,  0, -9,
 -5,  0,  3,
  5,  0,  3,
  7, 12,  0,
 -7, 12,  0,
 -2, 11,  2,
  2, 11,  2,
 -2, 10,  9,
  2, 10,  9,
  3,  5,  7,
 -3,  5,  7,
 -2, 16,  8,
  3, 15,  4,
 -3, 15,  4,
  2, 16,  8,
  5,  0, -1,
 13,  0,  4,
 14,  3, 11,
 13,  0, 13,
 -2,  4,  4,
-13,  7,  5,
-11,  0,  6,
 -5,  0, -1,
-13,  0,  4,
-14,  3, 11,
-13,  0, 13,
-15,  0, 12,
48,
3, 7, 0, 0, 4, 0, 0, 8, 0, 0, 2, // Right torso
3, 9, 0, 0, 5, 0, 0, 6, 0, 0, 0, // Left torso
3, 9, 0, 0, 10, 0, 0, 11, 0, 0, 7, // Back shoulders
3, 11, 0, 0, 8, 0, 0, 9, 0, 0, 7, // Back shoulders
3, 6, 0, 0, 7, 0, 0, 24, 0, 0, 2, // Chest
3, 24, 0, 0, 7, 0, 0, 1, 0, 0, 2, // Chest
3, 9, 0, 0, 6, 0, 0, 10, 0, 0, 2, // Left shoulder
3, 10, 0, 0, 6, 0, 0, 24, 0, 0, 2, // Left shoulder
3, 7, 0, 0, 11, 0, 0, 1, 0, 0, 2, // Right shoulder
3, 11, 0, 0, 7, 0, 0, 8, 0, 0, 2, // Right shoulder
3, 22, 0, 0, 2, 0, 0, 3, 0, 0, 9, // Right lower arm front
3, 23, 0, 0, 22, 0, 0, 3, 0, 0, 9, // Right lower arm front
3, 0, 0, 0, 21, 0, 0, 22, 0, 0, 9, // Right lower arm back
3, 22, 0, 0, 21, 0, 0, 2, 0, 0, 9, // Right lower arm back
3, 3, 0, 0, 8, 0, 0, 7, 0, 0, 2, // Right upper arm front
3, 3, 0, 0, 2, 0, 0, 8, 0, 0, 2, // Right upper arm front
3, 20, 0, 0, 8, 0, 0, 21, 0, 0, 0, // Right upper arm back
3, 21, 0, 0, 8, 0, 0, 2, 0, 0, 0, // Right upper arm back
3, 23, 0, 0, 0, 0, 0, 22, 0, 0, 9, // Right hand
3, 26, 0, 0, 25, 0, 0, 29, 0, 0, 9, // Left lower arm front
3, 26, 0, 0, 29, 0, 0, 30, 0, 0, 9, // Left lower arm front
3, 29, 0, 0, 28, 0, 0, 31, 0, 0, 9, // Left lower arm back
3, 25, 0, 0, 28, 0, 0, 29, 0, 0, 9, // Left lower arm back
3, 6, 0, 0, 9, 0, 0, 26, 0, 0, 2, // Left upper arm front
3, 9, 0, 0, 25, 0, 0, 26, 0, 0, 2, // Left upper arm front
3, 28, 0, 0, 9, 0, 0, 27, 0, 0, 7, // Left upper arm back
3, 25, 0, 0, 9, 0, 0, 28, 0, 0, 7, // Left upper arm back
3, 29, 0, 0, 31, 0, 0, 30, 0, 0, 9, // Left hand
3, 24, 0, 0, 1, 0, 0, 14, 0, 0, 9, // Face
3, 14, 0, 0, 15, 0, 0, 24, 0, 0, 9, // Face
3, 13, 0, 0, 12, 0, 0, 15, 0, 0, 9, // Forehead
3, 15, 0, 0, 14, 0, 0, 13, 0, 0, 9, // Forehead
3, 10, 0, 0, 24, 0, 0, 15, 0, 0, 9, // Left cheek
3, 15, 0, 0, 12, 0, 0, 10, 0, 0, 9, // Left cheek
3, 14, 0, 0, 1, 0, 0, 11, 0, 0, 9, // Right cheek
3, 14, 0, 0, 11, 0, 0, 13, 0, 0, 9, // Right cheek
3, 17, 0, 0, 18, 0, 0, 16, 0, 0, 0, // Head back
3, 16, 0, 0, 19, 0, 0, 17, 0, 0, 0, // Head back
3, 16, 0, 0, 12, 0, 0, 13, 0, 0, 0, // Head top
3, 13, 0, 0, 19, 0, 0, 16, 0, 0, 0, // Head top
3, 11, 0, 0, 10, 0, 0, 18, 0, 0, 0, // Neck back
3, 18, 0, 0, 17, 0, 0, 11, 0, 0, 0, // Neck back
3, 17, 0, 0, 19, 0, 0, 13, 0, 0, 0, // Right hair
3, 17, 0, 0, 13, 0, 0, 11, 0, 0, 0, // Right hair
3, 12, 0, 0, 16, 0, 0, 18, 0, 0, 0, // Left hair
3, 18, 0, 0, 10, 0, 0, 12, 0, 0, 0, // Left hair
3, 4, 0, 0, 5, 0, 0, 8, 0, 0, 7, // Back torso
3, 5, 0, 0, 9, 0, 0, 8, 0, 0, 7 // Back torso
};

const s8 timed_explosives[] =
{
  1,              // Number of vertex lists (Animation Frames)
  20,              // Number of verices per list.
   7, -14,  4, // 0 Right back bottom
   7, -14, -4, // 1 Right front bottom
  -7, -14, -4, // 2 Left front bottom
  -7, -14,  4, // 3 Left back bottom

   7,  14,  4, // 4 Right back top
   7,  14, -4, // 5 Right front top
  -7,  14, -4, // 6 Left front top
  -7,  14,  4, // 7 Left back top

   0,   9, -4, // 8 Diamond top
   5,   0, -4, // 9 Diamond right
   0,  -9, -4, // 10 Diamond bottom
  -5,   0, -4, // 11 Diamond left

  -7,   7, -4, // 12 Left stripe upper
  -7,   0, -4, // 13 Left stripe middle
  -7,  -7, -4, // 14 Left stripe lower

   7,   7, -4, // 15 Right stripe upper
   7,   0, -4, // 16 Right stripe middle
   7,  -7, -4, // 17 Right stripe lower

  -4,  20,  2, // 18 Left handle
   4,  20, -2, // 19 Right handle

  16,              // Number of faces in the polygon mesh.
                  // Each face is (NPoints, VID, U, V) + texture id.
  4, 4, 0, 0, 5, 0, 0, 6, 0, 0, 7, 0, 0, 9, // Top
  4, 2, 0, 0, 6, 0, 0, 5, 0, 0, 1, 0, 0, 7, // Front
  4, 1, 0, 0, 5, 0, 0, 4, 0, 0, 0, 0, 0, 2, // Right side
  4, 3, 0, 0, 7, 0, 0, 6, 0, 0, 2, 0, 0, 2, // Left side
  4, 8, 0, 0, 9, 0, 0, 10, 0, 0, 11, 0, 0, 9, // Diamond
  2, 6, 0, 0, 12, 0, 0, 0, // Left stripe black
  2, 12, 0, 0, 13, 0, 0, 3, // Left stripe yellow
  2, 13, 0, 0, 14, 0, 0, 0, // Left stripe black
  2, 14, 0, 0, 2, 0, 0, 3, // Left stripe yellow
  2, 5, 0, 0, 15, 0, 0, 3, // Right stripe yellow
  2, 15, 0, 0, 16, 0, 0, 0, // Right stripe black
  2, 16, 0, 0, 17, 0, 0, 3, // Right stripe yellow
  2, 17, 0, 0, 1, 0, 0, 0, // Right stripe black
  2, 7, 0, 0, 18, 0, 0, 2, // Handle left
  2, 18, 0, 0, 19, 0, 0, 2, // Handle top
  2, 19, 0, 0, 5, 0, 0, 2 // Handle right
};

const s8 timed_explosion[] =
{
  1,              // Number of vertex lists (Animation Frames)
  9,              // Number of verices per list.
    50, -60,   50, // 0 Right back bottom
    50, -60,  -50, // 1 Right front bottom
   -50, -60,  -50, // 2 Left front bottom
   -50, -60,   50, // 3 Left back bottom

   105,  87,    0, // 4 Right flame
     0,  87, -105, // 5 Front flame
  -105,  87,    0, // 6 Left flame
     0,  87,  105, // 7 Back flame

     0, 127,    0, // 8 Center flame

  13,              // Number of faces in the polygon mesh.
                  // Each face is (NPoints, VID, U, V) + texture id.
  4, 0, 0, 0, 1, 0, 0, 2, 0, 0, 3, 0, 0, 9, // Base
  3, 0, 0, 0, 4, 0, 0, 1, 0, 0, 7, // Right flame inside
  3, 1, 0, 0, 5, 0, 0, 2, 0, 0, 9, // Front flame inside
  3, 2, 0, 0, 6, 0, 0, 3, 0, 0, 9, // Left flame inside
  3, 3, 0, 0, 7, 0, 0, 0, 0, 0, 7, // Back flame inside
  3, 2, 0, 0, 8, 0, 0, 0, 0, 0, 7, // Center / flame front
  3, 0, 0, 0, 8, 0, 0, 2, 0, 0, 9, // Center / flame back
  3, 3, 0, 0, 8, 0, 0, 1, 0, 0, 9, // Center \ flame front
  3, 1, 0, 0, 8, 0, 0, 3, 0, 0, 9, // Center \ flame back
  3, 1, 0, 0, 4, 0, 0, 0, 0, 0, 2, // Right flame outside
  3, 2, 0, 0, 5, 0, 0, 1, 0, 0, 7, // Front flame outside
  3, 3, 0, 0, 6, 0, 0, 2, 0, 0, 2, // Left flame outside
  3, 0, 0, 0, 7, 0, 0, 3, 0, 0, 7 // Back flame outside
};

const s8 rocket_boat[] =
{
1,
24,
 27, -28,  79,
 27, -28, -67,
 27,  16, -73,
 27,  30,  -6,
 21,  17, -67,
 21,  31,  -6,
 27, -28,  30,
 27,  16,  35,
 20,  56,  21,
-20, -16, -63,
 21, -16, -63,
-27, -28, -67,
-27,  16, -73,
 21, -16,  -6,
-20, -16,  -6,
-27,  16,  35,
-20,  56,  21,
-20,  31,  -6,
-27,  30,  -6,
-20,  17, -67,
-27, -28,  30,
  0,  15,  79,
 20,  15,  69,
-19,  15,  69,
38,
3, 14, 0, 0, 13, 0, 0, 10, 0, 0, 9, // Floor
3, 10, 0, 0, 9, 0, 0, 14, 0, 0, 9, // Floor
3, 10, 0, 0, 4, 0, 0, 9, 0, 0, 2, // Rear wall
3, 4, 0, 0, 19, 0, 0, 9, 0, 0, 2, // Rear wall
3, 13, 0, 0, 4, 0, 0, 10, 0, 0, 7, // Right wall
3, 4, 0, 0, 13, 0, 0, 5, 0, 0, 7, // Right wall
3, 14, 0, 0, 9, 0, 0, 19, 0, 0, 7, // Left wall
3, 14, 0, 0, 19, 0, 0, 17, 0, 0, 7, // Left wall
3, 21, 0, 0, 22, 0, 0, 23, 0, 0, 8, // Hood front
3, 22, 0, 0, 7, 0, 0, 23, 0, 0, 8, // Hood
3, 23, 0, 0, 7, 0, 0, 15, 0, 0, 8, // Hood
3, 17, 0, 0, 5, 0, 0, 13, 0, 0, 2, // Front wall
3, 17, 0, 0, 13, 0, 0, 14, 0, 0, 2, // Front wall
3, 2, 0, 0, 12, 0, 0, 4, 0, 0, 8, // Rear ledge
3, 12, 0, 0, 19, 0, 0, 4, 0, 0, 8, // Rear ledge
3, 2, 0, 0, 1, 0, 0, 11, 0, 0, 2, // Rear
3, 11, 0, 0, 12, 0, 0, 2, 0, 0, 2, // Rear
3, 5, 0, 0, 2, 0, 0, 4, 0, 0, 8, // Right ledge
3, 3, 0, 0, 2, 0, 0, 5, 0, 0, 8, // Right ledge
3, 7, 0, 0, 3, 0, 0, 8, 0, 0, 7, // Right window
3, 6, 0, 0, 3, 0, 0, 7, 0, 0, 7, // Right window bottom
3, 1, 0, 0, 3, 0, 0, 6, 0, 0, 7, // Right side
3, 1, 0, 0, 2, 0, 0, 3, 0, 0, 7, // Right side rear
3, 19, 0, 0, 12, 0, 0, 18, 0, 0, 8, // Left ledge
3, 18, 0, 0, 17, 0, 0, 19, 0, 0, 8, // Left ledge
3, 16, 0, 0, 18, 0, 0, 15, 0, 0, 7, // Left window
3, 15, 0, 0, 18, 0, 0, 20, 0, 0, 7, // Left window bottom
3, 20, 0, 0, 18, 0, 0, 11, 0, 0, 7, // Left side
3, 18, 0, 0, 12, 0, 0, 11, 0, 0, 7, // Left side rear
3, 20, 0, 0, 6, 0, 0, 21, 0, 0, 0, // Front underside
3, 7, 0, 0, 22, 0, 0, 6, 0, 0, 2, // Right side front
3, 22, 0, 0, 21, 0, 0, 6, 0, 0, 2, // Right front
3, 20, 0, 0, 23, 0, 0, 15, 0, 0, 2, // Left side front
3, 20, 0, 0, 21, 0, 0, 23, 0, 0, 2, // Left front
3, 16, 0, 0, 8, 0, 0, 3, 0, 0, 9, // Front window inside
3, 16, 0, 0, 3, 0, 0, 18, 0, 0, 9, // Front window inside
3, 8, 0, 0, 15, 0, 0, 7, 0, 0, 0, // Front window
3, 16, 0, 0, 15, 0, 0, 8, 0, 0, 0 // Front window

};

/******************************************************************************/

void entity_move_forward(entity_t* e)
{
  e->xx = (fixsin(f2i(e->t)) * e->v) >> 6;
  e->yy = (fixcos(f2i(e->t)) * e->v) >> 6;
}

void entity_move_backwards(entity_t* e)
{
  e->xx = (fixsin(f2i(e->t)) * e->v) >> 6;
  e->yy = (fixcos(f2i(e->t)) * e->v) >> 6;
}

void entity_strafe_right(entity_t* e)
{
  e->xx += fixsin(f2i(e->t)+500) >> 4;
  e->yy += fixcos(f2i(e->t)+500) >> 4;
}

void entity_strafe_left(entity_t* e)
{
  e->xx -= fixsin(f2i(e->t)+500) >> 4;
  e->yy -= fixcos(f2i(e->t)+500) >> 4;
}

void entity_turn_right(entity_t* e)
{
  e->tt += i2f(25);
}

void entity_turn_left(entity_t* e)
{
  e->tt -= i2f(25);
}

void entity_turn_towards(entity_t* e, int x, int y)
{
  int x1 = x - (e->x + fixsin(f2i(e->t + i2f(100))));
  int y1 = y - (e->y + fixcos(f2i(e->t + i2f(100))));
  int x2 = x - (e->x + fixsin(f2i(e->t - i2f(100))));
  int y2 = y - (e->y + fixcos(f2i(e->t - i2f(100))));

  if ((x1 * x1 + y1 * y1) < (x2 * x2 + y2 * y2))
  {
    entity_turn_right(e);
  }
  else
  {
    entity_turn_left(e);
  }
}

void entity_kill(entity_t* e)
{
  engine_t* engine = e->engine;

  e->ontick = 0;
  e->onhit  = 0;
  e->visual.data = 0;
  e->radius = 0;
  binary_tree_delete(&(engine->root), e);
}

/******************************************************************************/

void entity_default(entity_t* e, int isjumping, int iscrawling)
{
  engine_t* engine = e->engine;

  {
    rom_cell_t* cell = &engine->curr_map->cells[f2i(e->y)][f2i(e->x)];
    int bot = cell->bot + (iscrawling ? engine->game.crawling : engine->game.walking);
    int top = cell->top - engine->game.ceiling;

    if (e->z <= bot)
    {
      e->z  -= ((e->z - bot) >> 2);
      e->zz -= (e->zz >> 2);
      if (isjumping && e->z < top)
      {
        e->z = bot;
        e->zz += engine->game.jumping;
      }
    }
    else
    {
      e->zz += engine->game.gravity;
    }

    if (e->z >= top)
    {
      e->z -= ((e->z - top) >> 2);
    }
  }
  if (e->life < 0) entity_kill(e);
}

void camera_behaviour(entity_t* e)
{
  engine_t* engine = e->engine;
  h_cell_t *hc0, *hc1, *hc2, *hc3;
  int x1,x2;

  if (engine->keyboard.pad & K_LT) e->p -= i2f(20);
  if (engine->keyboard.pad & K_RT) e->p += i2f(20);

  if (engine->camera_zoom_to > engine->camera_zoom) engine->camera_zoom += 20;
  else if (engine->camera_zoom_to < engine->camera_zoom) engine->camera_zoom -= 20;

  e->x = engine->main_entity->x;
  e->y = engine->main_entity->y - engine->camera_zoom;

  hc0 = (h_cell_t*)&huntercells[engine->grid->cells[f2i(e->y)][f2i(e->x)]];
  hc1 = (h_cell_t*)&huntercells[engine->grid->cells[f2i(e->y)][f2i(e->x)+1]];
  hc2 = (h_cell_t*)&huntercells[engine->grid->cells[f2i(e->y)-1][f2i(e->x)+1]];
  hc3 = (h_cell_t*)&huntercells[engine->grid->cells[f2i(e->y)-1][f2i(e->x)]];
  
  x1 = hc3->height + ((e->y & 255) * (hc0->height - hc3->height) >> 8);
  x2 = hc2->height + ((e->y & 255) * (hc1->height - hc2->height) >> 8);
  e->z = 110 + x1 + ((e->x & 255) * (x2 - x1) >> 8);

  e->p = i2f(40);
}

void box_behaviour(entity_t* e)
{
  e->t += i2f(25);
}

void vehicle_behaviour_auto(entity_t* e)
{
  engine_t* engine = e->engine;
  h_cell_t *hc0, *hc1, *hc2, *hc3;
  int x1,x2,y1,y2;
  
  entity_move_forward(e);

  hc0 = (h_cell_t*)&huntercells[engine->grid->cells[f2i(e->y + e->yy)][f2i(e->x + e->xx)]];
  hc1 = (h_cell_t*)&huntercells[engine->grid->cells[f2i(e->y + e->yy)][f2i(e->x + e->xx)+1]];
  hc2 = (h_cell_t*)&huntercells[engine->grid->cells[f2i(e->y + e->yy)-1][f2i(e->x + e->xx)+1]];
  hc3 = (h_cell_t*)&huntercells[engine->grid->cells[f2i(e->y + e->yy)-1][f2i(e->x + e->xx)]];
  
  x1 = hc3->height + (((e->y + e->yy) & 255) * (hc0->height - hc3->height) >> 8);
  x2 = hc2->height + (((e->y + e->yy) & 255) * (hc1->height - hc2->height) >> 8);

  y1 = hc3->height + (((e->x + e->xx) & 255) * (hc2->height - hc3->height) >> 8);
  y2 = hc0->height + (((e->x + e->xx) & 255) * (hc1->height - hc0->height) >> 8);

  e->z = x1 + (((e->x + e->xx) & 255) * (x2 - x1) >> 8) - ((s8*)e->visual.data)[3];
  e->p = (x1 - x2) * fixsin(f2i(e->t)) + (y1 - y2) * fixcos(f2i(e->t));
}

void player_collision(entity_t* e1, entity_t* e2);

void timed_explosion_behaviour(entity_t* e)
{
  e->t += i2f(200);
  e->life -= 15;

  if (e->life < 0)
    entity_kill(e);
}

void timed_explosion_hit(entity_t* e, u16 rom_entity_ID)
{
  engine_t* engine = e->engine;

  if (engine->building_strength[rom_entity_ID] > 60)
    engine->building_strength[rom_entity_ID] -= 60;
  else
    engine->building_strength[rom_entity_ID] = 0;

  e->onBGhit = 0;
}

void timed_explosives_behaviour(entity_t* e)
{
  e->life -= 5;

  if (e->life < 0) {
    e->visual.data = &timed_explosion;
    e->ontick = timed_explosion_behaviour;
    e->onBGhit = timed_explosion_hit;
    e->z += 46;
    e->xx = 1;
    e->life = 127;
    e->radius = 50;
  }
}

void walking_behaviour(entity_t* e)
{
  engine_t* engine = e->engine;
  h_cell_t *hc0, *hc1, *hc2, *hc3;
  int x1,x2;

  if (engine->keyboard.pad & K_LPAD) {
    if (engine->keyboard.pad & K_LL) {
      if (engine->keyboard.pad & K_LU)
        e->t = i2f(1792);
      else if (engine->keyboard.pad & K_LD)
        e->t = i2f(1280);
      else
        e->t = i2f(1536);
    }
    else if (engine->keyboard.pad & K_LR) {
      if (engine->keyboard.pad & K_LU)
        e->t = i2f(256);
      else if (engine->keyboard.pad & K_LD)
        e->t = i2f(768);
      else
        e->t = i2f(512);
    }
    else if (engine->keyboard.pad & K_LD)
      e->t = i2f(1024);
    else
      e->t = 0;

    e->v = 2;    
  }
  else
    e->v = 0;

  entity_move_forward(e);

  if ((engine->keyboard.pad & K_LT) && !(engine->keyboard._pad & K_LT) )
    engine->mapon = engine->mapon ? 0 : 1;

  if ((engine->keyboard.pad & K_A) && !(engine->keyboard._pad & K_A) ) {
    entity_t* et = engine_entity(engine, e->x, e->y, e->z - 30, 0);
    et->visual.data = &timed_explosives;
    et->ontick = timed_explosives_behaviour;
    et->radius = 0;
  }

  if ((engine->keyboard.pad & K_B) && !(engine->keyboard._pad & K_B) ) {
    if (engine->building_strength[20] > 0)
      engine->building_strength[20] = 0;
    else
      engine->building_strength[20] = 100;
  }

  hc0 = (h_cell_t*)&huntercells[engine->grid->cells[f2i(e->y + e->yy)][f2i(e->x + e->xx)]];
  hc1 = (h_cell_t*)&huntercells[engine->grid->cells[f2i(e->y + e->yy)][f2i(e->x + e->xx)+1]];
  hc2 = (h_cell_t*)&huntercells[engine->grid->cells[f2i(e->y + e->yy)-1][f2i(e->x + e->xx)+1]];
  hc3 = (h_cell_t*)&huntercells[engine->grid->cells[f2i(e->y + e->yy)-1][f2i(e->x + e->xx)]];
  
  x1 = hc3->height + (((e->y + e->yy) & 255) * (hc0->height - hc3->height) >> 8);
  x2 = hc2->height + (((e->y + e->yy) & 255) * (hc1->height - hc2->height) >> 8);

  if (hc0->color == 1)
    e->visual.data = &swimmer;
  else switch (hc0->style)
  {
    case 4: e->visual.data = &player; break;
    case 0: e->visual.data = ((e->x + e->xx) & 0xff) < ((e->y + e->yy) & 0xff) ? (void*)&swimmer : (void*)&player; break;
    case 1: e->visual.data = ((e->x + e->xx) & 0xff) > 255-((e->y + e->yy) & 0xff) ? (void*)&swimmer : (void*)&player; break;
    case 2: e->visual.data = ((e->x + e->xx) & 0xff) > ((e->y + e->yy) & 0xff) ? (void*)&swimmer : (void*)&player; break;
    case 3: e->visual.data = ((e->x + e->xx) & 0xff) < 255-((e->y + e->yy) & 0xff) ? (void*)&swimmer : (void*)&player; break;
  }

  e->z = x1 + (((e->x + e->xx) & 255) * (x2 - x1) >> 8) - ((s8*)e->visual.data)[3];
}

void vehicle_behaviour(entity_t* e)
{
  engine_t* engine = e->engine;
  h_cell_t *hc0, *hc1, *hc2, *hc3;
  int x1,x2,y1,y2;
  
  if (engine->keyboard.pad & K_LL) {e->t -= i2f(30); entity_move_forward(e);} else
  if (engine->keyboard.pad & K_LR) {e->t +=  i2f(30); entity_move_forward(e);}
  if (engine->keyboard.pad & K_LU && e->v < 5) {e->v += 1; entity_move_forward(e);} else
  if (engine->keyboard.pad & K_LD && e->v > 0) {e->v -= 1; entity_move_forward(e);}

  if ((engine->keyboard.pad & K_LT) && !(engine->keyboard._pad & K_LT) )
    engine->mapon = engine->mapon ? 0 : 1;

  if ((engine->keyboard.pad & K_RT) && !(engine->keyboard._pad & K_RT) ) {
    entity_t* et = engine_entity(engine, e->x + e->radius + 10, e->y, 1000, walking_behaviour);
    et->visual.data = &player;
    et->t = e->t;
    et->radius = 9;
    et->onhit = player_collision;
    engine->main_entity = et;
    engine->camera_zoom_to = 180;
    e->ontick = vehicle_behaviour_auto;
  }

  hc0 = (h_cell_t*)&huntercells[engine->grid->cells[f2i(e->y + e->yy)][f2i(e->x + e->xx)]];
  hc1 = (h_cell_t*)&huntercells[engine->grid->cells[f2i(e->y + e->yy)][f2i(e->x + e->xx)+1]];
  hc2 = (h_cell_t*)&huntercells[engine->grid->cells[f2i(e->y + e->yy)-1][f2i(e->x + e->xx)+1]];
  hc3 = (h_cell_t*)&huntercells[engine->grid->cells[f2i(e->y + e->yy)-1][f2i(e->x + e->xx)]];
  
  x1 = hc3->height + (((e->y + e->yy) & 255) * (hc0->height - hc3->height) >> 8);
  x2 = hc2->height + (((e->y + e->yy) & 255) * (hc1->height - hc2->height) >> 8);

  y1 = hc3->height + (((e->x + e->xx) & 255) * (hc2->height - hc3->height) >> 8);
  y2 = hc0->height + (((e->x + e->xx) & 255) * (hc1->height - hc0->height) >> 8);

  e->z = x1 + (((e->x + e->xx) & 255) * (x2 - x1) >> 8) - ((s8*)e->visual.data)[3];
  e->p = (x1 - x2) * fixsin(f2i(e->t)) + (y1 - y2) * fixcos(f2i(e->t));
}

void player_collision(entity_t* e1, entity_t* e2)
{
  engine_t* engine = e1->engine;

  if (e2->visual.data == &car || e2->visual.data == &jeep || e2->visual.data == &rocket_boat) {
    engine->main_entity = e2;
    engine->camera_zoom_to = 320;
    e2->ontick = vehicle_behaviour;
    binary_tree_delete(&(engine->root), e1);
  }
}

void game_init(engine_t* engine)
{
  int x, y;
  entity_t* enti;

  engine_load_map(engine, (rom_map_t*)&e1m1, (h_map_t*)&huntermap);

  engine->camera->p = i2f(40);
  engine->camera_zoom = 600;
  engine->camera_zoom_to = 180;

  enti = engine_entity(engine, i2f(113)+128, i2f(15)+128, i2f(0), walking_behaviour);
  enti->visual.data = &player;
  enti->t = i2f(1280);
  enti->radius = 9;
  enti->onhit = player_collision;
  engine->main_entity = enti;
  engine->currModel = 1;

  enti = engine_entity(engine, i2f(112)+210, i2f(16)+50, i2f(0), vehicle_behaviour_auto);
  enti->visual.data = &car;
  enti->t = i2f(880);
  enti->radius = 33;

  enti = engine_entity(engine, i2f(112)+70, i2f(16)+128, i2f(0), vehicle_behaviour_auto);
  enti->visual.data = &jeep;
  enti->t = i2f(160);
  enti->radius = 39;

  enti = engine_entity(engine, i2f(114)+128, i2f(15)+100, i2f(0), vehicle_behaviour_auto);
  enti->visual.data = &rocket_boat;
  enti->t = i2f(1950);
  enti->radius = 69;
}

void game_draw(engine_t* engine)
{
  engine_draw(engine);

  if (engine->mapon)
    draw_map_overlay(engine);
}

void game_tick(engine_t* engine)
{
  engine_tick(engine);
}

void game_loop(engine_t* engine)
{
  game_tick(engine);
  game_draw(engine);
}

