#include "engine.h"

#include "include\world.h"
#include "include\mem.h"
#include "include\object.h"
#include "include\joypad.h"
#include "include\asm.h"
#include "include\interrupts.h"
#include "include\timer.h"

#include "chhud.h"
#include "mphud.h"
#include "font_pc_bios.h"

#include "bgadjustment.h"
#include "chadjustment.h"
#include "mpprecaution.h"
#include "chtitle.h"
#include "mptitle.h"

engine_t engine;

int min(int a, int b)
{
  return MIN(a, b);
}

int max(int a, int b)
{
  return MAX(a, b);
}

const s16 moon_offset[20] = {0,732,1302,1302,0,150,0,1373,0,125,125,1820,0,512,1230,1230,0,80,0,0};

void set_moon_hbias()
{
  u8 i;
  s16* param = (s16*)((WA[28].param << 1) + 0x00020000);

  for (i = 0; i < 20; i++) {
    if (WA[28].gy + i < WA[29].gy)
      *param = 500;
    else if (moon_offset[i] > 0)
      *param = f2i(fixsin((engine.tick << 6)+moon_offset[i]) << 3);
    else
      *param = 0;

    *(param+1) = *param;
    param += 2;
  }
}

void vb_init(void)
{ u16* txtinit;
  u16* const param = (u16*)0x0003C000;

  //Precaution screen
  copymem((void*)CharSeg3,(void*)FONT,512*16);
  copymem((void*)BGMap(0),(void*)MPPRECAUTION,8192); // 64*64*2
  vbSetWorld(31, WRLD_ON, 0, 0, 0, 0, 0, 0, 384, 224);
  vbSetWorld(30,WRLD_END, 0, 0, 0, 0, 0, 0, 0, 0);

  vbDisplayOn();
  vbFXFadeIn(1);
  while (!(vbReadPad()&(K_ANY)));
  vbFXFadeOut(1);

  //IPD screen
  copymem((void*)CharSeg0, (void*)CHADJUSTMENT, 256*16);
  copymem((void*)BGMap(0), (void*)BGADJUSTMENT, 450*16);
  vbSetWorld(31, WRLD_LON|WRLD_OVR, 0, 0, 0, 0, 0, 0, 384, 224);
  vbSetWorld(30, WRLD_RON|WRLD_OVR, 0, 0, 0, 0, 0, 224, 384, 224);
  vbSetWorld(29, WRLD_END, 0, 0, 0, 0, 0, 0, 0, 0);

  vbFXFadeIn(1);
  while (!(vbReadPad()&(K_ANY)));
  vbFXFadeOut(1);

  //Title screen
  copymem((void*)CharSeg0,(void*)CHTITLE,512*16);
  copymem((void*)BGMap(0),(void*)MPTITLE,8192); // 64*64*2
  vbSetWorld(31, WRLD_ON, 0, 0, 0, 0, 0, 0, 384, 224);
  vbSetWorld(30,WRLD_END, 0, 0, 0, 0, 0, 0, 0, 0);

  vbFXFadeIn(1);
  while (!(vbReadPad()&(K_ANY)));
  vbFXFadeOut(1);

  //Game setup
  copymem((void*)CharSeg0,(void*)CHHUD,512*16);
  copymem((void*)BGMap(0),(void*)MPHUD,8192); // 64*64*2

  vbSetWorld(31,WRLD_ON,0,6,0,0,0,152,384,100); // Stars
  vbSetWorld(30,WRLD_ON,120,3,66,392,0,464,27,19); // Moon
  vbSetWorld(29,WRLD_ON,0,0,91,0,0,288,384,224); // Ocean
  vbSetWorld(28,WRLD_ON|WRLD_HBIAS|WRLD_OVR,120,3,98,392,0,492,36,19); // Moon reflection
  vbSetWorld(27,WRLD_ON,0,0,189,0,0,40,384,35); // HUD mask
  vbSetWorld(26,WRLD_ON,32,-12,189,0,0,0,320,40); // HUD
  vbSetWorld(25,WRLD_ON|WRLD_OBJ,0,0,0,0,0,0,0,0);
  vbSetWorld(24,WRLD_ON|1,0,0,0,0,0,0,384,224); // Debug text
  vbSetWorld(23,WRLD_END, 0, 0, 0, 0, 0, 0, 0, 0);
  for (txtinit = &BGMM[0x1000]; txtinit != &BGMM[0x2000]; txtinit++)
    *txtinit = 1536;

  WA[28].param = (u32)param;
  WA[28].ovr = 3952;

  vbDisplayOn();
  vbDisplayShow();

  VIP_REGS[GPLT1] = 0x90;

  HW_REGS[WCR] = 1;
}

void vb_flip(void)
{
  u32* temp;
  static u16 pgflip = XPBSY1;

  engine.keyboard.pad = vbReadPad();

  // Flip the frame buffers.

  pgflip = ~pgflip & 0x000C;
  VIP_REGS[XPCTRL] = VIP_REGS[XPSTTS] | XPEN; // sets the XPEN bit to refresh the screen
  while (!(VIP_REGS[XPSTTS] & pgflip)); // makes sure to freeze screen on framebuffer 1
  VIP_REGS[XPCTRL] = VIP_REGS[XPSTTS] & ~XPEN; // turn screen refreshing off again

  temp = engine.scrin;
  engine.scrin = engine.baffer;
  engine.baffer = temp;
}

int main()
{
  u16 real = 0;
  s32 camXOffset;

  vb_init();
  engine_init(&engine, (framebuffer_t*) 0x00000000, (framebuffer_t*) 0x00000000);
  engine.scrin  = (u32*)0x00000000;
  engine.baffer = (u32*)0x00008000;
  game_init(&engine);

  while (1/*!engine.keyboard.select*/)
  {
    game_tick(&engine);

    ((u16*)BGMap(0))[68] = 144 + (engine.main_entity->x >> 7)/100;
    ((u16*)BGMap(0))[69] = 144 + ((engine.main_entity->x >> 7)%100)/10;
    ((u16*)BGMap(0))[70] = 144 + (engine.main_entity->x >> 7)%10;

    ((u16*)BGMap(0))[196] = 144 + (engine.main_entity->y >> 7)/100;
    ((u16*)BGMap(0))[197] = 144 + ((engine.main_entity->y >> 7)%100)/10;
    ((u16*)BGMap(0))[198] = 144 + (engine.main_entity->y >> 7)%10;

    WA[29].gy = 91 + (engine.camera->z - 110 >> 4);
    WA[29].my = WA[29].gy & 1 ? 289 : 288;
    WA[31].gy = WA[29].gy - 100;
    WA[30].gx = 180 + f2i(fixcos(engine.tick) * 220);
    WA[30].gy = WA[29].gy + 132 + f2i(fixsin(engine.tick) * 220);
    WA[28].gx = WA[30].gx;
    WA[28].gy = WA[29].gy + (WA[29].gy-WA[30].gy) - 20;
    set_moon_hbias();

    camXOffset = 16;

    engine.camera->x -= camXOffset;

    game_draw(&engine);

    engine.camera->x += camXOffset << 1;

    engine.baffer += 0x4000;
    game_draw(&engine);
    engine.baffer -= 0x4000;

    engine.camera->x -= camXOffset;


    vb_flip();
  }
  return 0;
}
